/* ********************************************** */
/*    About Dialogbox Procesure Class ver.1.00    */
/*                               'aboutprc.cpp'   */
/*                               by T.Nakayama    */
/* ********************************************** */

#include "aboutprc.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

DlgAboutProc :: DlgAboutProc()
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

DlgAboutProc :: ~DlgAboutProc()
{
}

/* ************************************************************************* */
/*    WM_PAINT                                                               */
/* ************************************************************************* */

int DlgAboutProc :: Paint(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	HDC			hDC;
	PAINTSTRUCT	ps;
	HFONT		hFont;
	HFONT		hDefFont;
	TEXTMETRIC	tm;
	LOGFONT		lf;

	// ======== Get a device context
	hDC = BeginPaint(hDlg, &ps);

	// Get GUI font metrics
	hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	hDefFont = (HFONT)SelectObject(hDC, hFont);
	GetTextMetrics(hDC, &tm);
	memset(&lf, 0, sizeof(lf));
	GetTextFace(hDC, LF_FACESIZE, lf.lfFaceName);
	SelectObject(hDC, hDefFont);

	// ======== Create font added under line
	lf.lfHeight = tm.tmHeight;
	lf.lfWidth = 0;
	lf.lfEscapement = 0;
	lf.lfOrientation = 0;
	lf.lfWeight = FW_DONTCARE;
	lf.lfItalic = tm.tmItalic;
	lf.lfUnderline = TRUE;
	lf.lfStrikeOut = tm.tmStruckOut;
	lf.lfCharSet = tm.tmCharSet;
	lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	lf.lfQuality = DEFAULT_QUALITY;
	lf.lfPitchAndFamily = tm.tmPitchAndFamily;
	hFont = CreateFontIndirect(&lf);
	hDefFont = (HFONT)SelectObject(hDC, hFont);

	// ======== Drow URL Text
	if(_bAboutURLHit) {
		SetTextColor(hDC, RGB(255, 0, 0));
	}
	else{
		SetTextColor(hDC, RGB(0, 0, 255));
	}
	SetBkColor(hDC, GetSysColor(COLOR_3DFACE));
	DrawText(hDC, _pszAboutURL, strlen(_pszAboutURL),
		&_AboutURLRect, DT_LEFT | DT_SINGLELINE);

	// ======== Put back font
	SelectObject(hDC, hDefFont);
	DeleteObject(hFont);

	// ======== Release device context
	EndPaint(hDlg, &ps);

	return(DIALOG_COMPLETE);
}

/* ************************************************************************* */
/*    WM_INITDIALOG Appendix                                                 */
/* ************************************************************************* */

int DlgAboutProc :: InitDialogAppendix(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	HWND		hWnd;
	RECT		rc;
	POINT		point;
	DWORD		dwSize, dwReserved;
	LPTSTR		pVersion;
	LPVOID		pInfo1, pInfo2;
	UINT		uLength;
	TCHAR		szBuf[STRING_BUFFER];
	TCHAR		szPreWord[STRING_BUFFER], szPostWord[STRING_BUFFER];
	TCHAR		szFileName[STRING_BUFFER];

	CenteringDialog(hDlg);

	// ======== Read a URL static text in resource
	memset(_pszAboutURL, 0, sizeof(_pszAboutURL));
	LoadString(_hInst, IDS_URL, _pszAboutURL, sizeof(_pszAboutURL));

	// ======== Read Text Words
	LoadString(_hInst, IDS_ABOUT_PRE_WORD, szPreWord, sizeof(szPreWord));
	LoadString(_hInst, IDS_ABOUT_POST_WORD, szPostWord, sizeof(szPostWord));

	// ======== Read File Name
	LoadString(_hInst, IDS_FILENAME, szFileName, STRING_BUFFER);

	// ======== Get client coordinate of IDC_URL
	hWnd = GetDlgItem(hDlg, IDC_URL);
	GetWindowRect(hWnd, &rc);

	point.x = rc.left;
	point.y = rc.top;
	ScreenToClient(hDlg, &point);
	_AboutURLRect.left = point.x;
	_AboutURLRect.top = point.y;

	point.x = rc.right;
	point.y = rc.bottom;
	ScreenToClient(hDlg, &point);
	_AboutURLRect.right = point.x;
	_AboutURLRect.bottom = point.y;

	// ======== Destroy IDC_URL child window
	DestroyWindow(hWnd);

	// ======== Initialize URL flag
	_bAboutURLHit = FALSE;

	// ======== Read & Set Version Information
	dwSize = GetFileVersionInfoSize(szFileName, &dwReserved);
	pVersion = new TCHAR[dwSize];
	if(pVersion && GetFileVersionInfo(szFileName, 0, dwSize, pVersion)){
		VerQueryValue(pVersion, "\\StringFileInfo\\040904E4\\LegalCopyright",
			&pInfo1, &uLength);
		SetWindowText(GetDlgItem(hDlg, IDC_COPYRIGHT), (LPCTSTR)pInfo1);
		VerQueryValue(pVersion, "\\StringFileInfo\\040904E4\\ProductName",
			&pInfo1, &uLength);
		VerQueryValue(pVersion, "\\StringFileInfo\\040904E4\\ProductVersion",
			&pInfo2, &uLength);
		wsprintf(szBuf, "%s ver.%s", (LPCTSTR)pInfo1, (LPCTSTR)pInfo2);
		SetWindowText(GetDlgItem(hDlg, IDC_PRODUCT), szBuf);
		wsprintf(szBuf, "%s%s%s", szPreWord, (LPCTSTR)pInfo1, szPostWord);
		SetWindowText(hDlg, szBuf);
	}
	_DELETEARRAY(pVersion);

	return(DIALOG_COMPLETE);
}

/* ************************************************************************* */
/*    WM_NCHITTEST                                                           */
/* ************************************************************************* */

int DlgAboutProc :: NcHitTest(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	BOOL	bFlag;
	POINT	point;

	point.x = LOWORD(lParam);
	point.y = HIWORD(lParam);
	ScreenToClient(hDlg, &point);
	bFlag = PtInRect(&_AboutURLRect, point);
	if (bFlag != _bAboutURLHit) {
		_bAboutURLHit = bFlag;
		InvalidateRect(hDlg, &_AboutURLRect, FALSE);
	}
	return(DIALOG_COMPLETE);
}

/* ************************************************************************* */
/*    WM_SETCURSOR                                                           */
/* ************************************************************************* */

int DlgAboutProc :: SetCursor(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	int		iRet = DIALOG_COMPLETE;
	HCURSOR	hCursor;

	if(_bAboutURLHit){
		// ======== Fail IDC_HAND, when a OS version isn't supported.
		hCursor = LoadCursor(NULL, MAKEINTRESOURCE(32649));
		if(!hCursor){
			hCursor = LoadCursor(NULL, IDC_IBEAM);
		}
		:: SetCursor(hCursor);

		// ======== if a left button is pushed, execute the URL.
		if((HIWORD(lParam) == WM_LBUTTONDOWN) ||
			(HIWORD(lParam) == WM_LBUTTONDBLCLK)){
			ShellExecute(hDlg, NULL, _pszAboutURL, NULL, NULL, SW_SHOWNORMAL);
		}
		iRet = DIALOG_ABOUT_EXECUTE_BROWSER;
	}
	return(iRet);
}

/* ************************************************************************* */
/*    Dialog Procedure                                                       */
/* ************************************************************************* */

BOOL DlgAboutProc :: DlgProcedure(
	HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg){
		case WM_COMMAND:	// ++++++++++++++++++++++++++++++++ WM_COMMAND ++++
			if(Command(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_PAINT:	// ++++++++++++++++++++++++++++++++++++++ WM_PAINT ++++
			if(Paint(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_INITDIALOG:	// +++++++++++++++++++++++++++++ WM_INITDIALOG ++++
			CenteringDialog(hDlg);
			if(InitDialog(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_NCHITTEST: // +++++++++++++++++++++++++++++++ WM_NCHITTEST ++++
			if(NcHitTest(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_SETCURSOR:	// ++++++++++++++++++++++++++++++ WM_SETCURSOR ++++
			if(SetCursor(hDlg, wParam, lParam) == DIALOG_ABOUT_EXECUTE_BROWSER)
				return(TRUE);
			break;
	}
	return(FALSE);
}

