/* ******************************** */
/*    Application Class ver.1.00    */
/*                      'appli.h'   */
/*                 by T.Nakayama    */
/* ********************************* */

#include "appli.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

Appli :: Appli()
{
	SetInitLine(APPLI_INITPLAYLINE);
	SetInitPlay(0, APPLI_INITPLAY00);
	SetInitPlay(1, APPLI_INITPLAY01);
	SetInitPlay(2, APPLI_INITPLAY02);
	SetInitPlay(3, APPLI_INITPLAY03);
	SetInitPlay(4, APPLI_INITPLAY04);
	SetInitPlay(5, APPLI_INITPLAY05);
	SetInitPlay(6, APPLI_INITPLAY06);
	SetInitPlay(7, APPLI_INITPLAY07);
	SetInitPlay(8, APPLI_INITPLAY08);
	SetInitPlay(9, APPLI_INITPLAY09);
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

Appli :: ~Appli()
{
}

/* ************************************************************************* */
/*    Read MIDI Data  :  MIDIf[^̓ǂݍ & SMFNX̍쐬              */
/* ************************************************************************* */

LPSmfData Appli :: ReadMidiData(LPBYTE pData, DWORD dwSize)
{
	LPSmfData	pSmfData = NULL;

	// Read MIDI Data
	pSmfData = new SmfData;
	if(!pSmfData->IsSmfData(pData, dwSize))
		pSmfData->ReadData(pData, dwSize);
	else
		_DELETE(pSmfData);

	return(pSmfData);
}

/* ************************************************************************* */
/*    Free MIDI Data  :  SMFNX̉                                     */
/* ************************************************************************* */

int Appli :: FreeMidiData(LPSmfData pSmfData)
{
	_DELETE(pSmfData);
	return(APPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Create MML Data  :  MMLNX̍쐬                                    */
/* ************************************************************************* */

inline LPMmlData Appli :: CreateMmlData(void)
{
	return(new MmlData);
}

/* ************************************************************************* */
/*    Free MML Data  :  MMLNX̉                                      */
/* ************************************************************************* */

int Appli :: FreeMmlData(LPMmlData pMmlData)
{
	_DELETE(pMmlData);
	return(APPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Convert MIDI Data to MML Data  :  MIDIf[^MMLf[^ւ̕ϊ      */
/* ************************************************************************* */

int Appli :: ConvertMidiToMml(LPSmfData pSmfData, LPMmlData pMmlData)
{
	LPSmf2Mml	pSmf2Mml;

	if(!pSmfData || !pMmlData)
		return(APPLI_NULLPOINTER);

	pSmf2Mml = new Smf2Mml;
	pSmf2Mml->Convert(pSmfData, pMmlData);
	_DELETE(pSmf2Mml);

	return(APPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Convert MIDI Data to MML Data  :  MIDIf[^MMLf[^ւ̕ϊ      */
/* ************************************************************************* */

LPMmlData Appli :: ConvertMidiToMml(LPSmfData pSmfData)
{
	LPMmlData	pMmlData;

	pMmlData = CreateMmlData();
	if(ConvertMidiToMml(pSmfData, pMmlData))
		FreeMmlData(pMmlData);

	return(pMmlData);
}

/* ************************************************************************* */
/*    Normalize  :  ŏ\֑                                 */
/* ************************************************************************* */

inline int Appli :: Normalize(LPMmlChannel pMmlChannel, int iMinDiv)
{
	return(pMmlChannel->Normalize(iMinDiv));
}

int Appli :: Normalize(LPMmlData pMmlData, int iMinDiv)
{
	int	i, iSurplus = 0;

	for(i = 0; i < pMmlData->GetChannelCount() ; i++)
		iSurplus += Normalize(pMmlData->GetMmlChannel(i), iMinDiv);
	return(iSurplus);
}

inline int Appli :: Normalize(LPMmlData pMmlData)
{
	return(Normalize(pMmlData, GetDivision(pMmlData) / MMLMINDIVRATIO));
}

/* ************************************************************************* */
/*    Divide  :  w肵Xebvŋ؂                                    */
/* ************************************************************************* */

inline int Appli :: Divide(LPMmlChannel pMmlChannel, int iBarTime)
{
	return(pMmlChannel->Divide(iBarTime));
}

int Appli :: Divide(LPMmlData pMmlData, int iBarTime)
{
	int	i, iBar, iBarMax = 0;

	for(i = 0; i < pMmlData->GetChannelCount() ; i++){
		iBar = Divide(pMmlData->GetMmlChannel(i), iBarTime);
		iBarMax = iBar > iBarMax ? iBar : iBarMax;
	}
	return(iBarMax);
}

/* ************************************************************************* */
/*    Convert Length  :  f^^Cɕϊ                        */
/* ************************************************************************* */

inline int Appli :: ConvertLength(LPMmlChannel pMmlChannel, int iDivision)
{
	return(pMmlChannel->ConvertLength(iDivision));
}

int Appli :: ConvertLength(LPMmlData pMmlData, int iDivision)
{
	int	i, iResult;

	for(i = 0; i < pMmlData->GetChannelCount() ; i++)
		iResult |= ConvertLength(pMmlData->GetMmlChannel(i), iDivision);
	return(iResult);
}

inline int Appli :: ConvertLength(LPMmlData pMmlData)
{
	return(ConvertLength(pMmlData, GetDivision(pMmlData)));
}

/* ************************************************************************* */
/*    Get Division  :  S̕\̎擾                                */
/* ************************************************************************* */

inline int Appli :: GetDivision(LPMmlData pMmlData)
{
	return(pMmlData->GetDivision());
}

inline int Appli :: GetDivision(LPSmfData pSmfData)
{
	return(pSmfData->GetDivision());
}

/* ************************************************************************* */
/*    Get Channel Count  :  gpĂ`l̎擾                     */
/* ************************************************************************* */

inline int Appli :: GetChannelCount(LPMmlData pMmlData)
{
	return(pMmlData->GetChannelCount());
}

/* ************************************************************************* */
/*    Get Bar Count  :  w`l̏ߐ̎擾                         */
/* ************************************************************************* */

inline int Appli :: GetBarCount(LPMmlData pMmlData, int iChannel)
{
	return(pMmlData->GetBarCount(iChannel));
}

inline int Appli :: GetBarCount(LPMmlChannel pMmlChannel)
{
	return(pMmlChannel->GetBarCount());
}

/* ************************************************************************* */
/*    Get Step Time  :  w`l̃Xebv^C̎擾                 */
/* ************************************************************************* */

inline DWORD Appli :: GetStepTime(LPMmlData pMmlData, int iChannel)
{
	return(pMmlData->GetStepTime(iChannel));
}

inline DWORD Appli :: GetStepTime(LPMmlChannel pMmlChannel)
{
	return(pMmlChannel->GetStepTime());
}

/* ************************************************************************* */
/*    Get Length Count  :  ̎gp̎擾                                */
/* ************************************************************************* */

inline int Appli :: GetLengthCount(
	LPMmlData pMmlData, int iChannel, int iLength)
{
	return(pMmlData->GetLengthCount(iChannel, iLength));
}

inline int Appli :: GetLengthCount(LPMmlChannel pMmlChannel, int iLength)
{
	return(pMmlChannel->GetLengthCount(iLength));
}

/* ************************************************************************* */
/*    Get Max Length Count  :  ł̎擾                            */
/* ************************************************************************* */

inline int Appli :: GetMaxLengthCount(LPMmlData pMmlData, int iChannel)
{
	return(pMmlData->GetMaxLengthCount(iChannel));
}

inline int Appli :: GetMaxLengthCount(LPMmlChannel pMmlChannel)
{
	return(pMmlChannel->GetMaxLengthCount());
}

/* ************************************************************************* */
/*    Create Source  :  EDPLAYɂBASICXg쐬                    */
/* ************************************************************************* */

string Appli :: CreateSource(LPMmlData pMmlData, LPEdTrans pEdTrans,
	int iChannelCount, int iLine, int iStep, LPTSTR pszPlay)
{
	int			i, iBar, iBarMax;
	string		source = "", text;

	for(i = iBarMax = iBar = 0; i < iChannelCount; i++){
		if(pEdTrans[i].GetChannel() >= 0){
			iBar = pMmlData->GetBarCount(pEdTrans[i].GetChannel());
			iBarMax = iBar > iBarMax ? iBar : iBarMax;
		}
	}

	for(i = 0; i < iChannelCount; i++)
		if(pEdTrans[i].GetChannel() >= 0)
			pEdTrans[i].InitIterator(pMmlData, pEdTrans[i].GetChannel());
	for(i = 0; i < iBarMax; i++, iLine += iStep){
		CreateSourceLine(pMmlData, pEdTrans, iChannelCount, iLine,
			text, i == 0 ? EDTRANS_INITMML : EDTRANS_NONE, pszPlay);
		source += text;
	}

	return(source);
}

/* ************************************************************************* */
/*    Create Source Line :  EDPLAYɂ1s쐬                        */
/* ************************************************************************* */

int Appli :: CreateSourceLine(LPMmlData pMmlData, LPEdTrans pEdTrans,
	int iChannelCount, int iLine, string &text, UINT uFlag, LPTSTR pszPlay)
{
	int			i;
	TCHAR		szTemp[STRING_BUFFER];
	TCHAR		szQuote[2] = {0x22, 0x00};
	TCHAR		szCRLF[3] = {0x0d, 0x0a, 0x00};
	string		mml;

	if(pszPlay)
		sprintf(szTemp, "%4d %s ", iLine, pszPlay);
	else
		sprintf(szTemp, "%4d %s ", iLine, APPLI_CMDPLAY);
	text = szTemp;
	for(i = 0; i < iChannelCount; i++){
		mml = "";
		if(pEdTrans[i].GetChannel() >= 0)
			pEdTrans[i].Translate(pMmlData, pEdTrans[i].GetChannel(),
				mml, uFlag);
		if(i)
			text += ",";
		text += szQuote;
		text += mml;
		text += szQuote;
	}
	text += szCRLF;

	return(text.length());
}

/* ************************************************************************* */
/*    Create Source Initialize :  EDPLAY̏BASICXg쐬        */
/* ************************************************************************* */

string Appli :: CreateSourceInit(int iLine, int iStep)
{
	int			i;
	string		source = "", temp;
	TCHAR		szTemp[STRING_BUFFER];

	for(i = 0; i < APPLI_INITPLAYLINE; i++, iLine += iStep){
		temp = GetInitPlay(i);
		sprintf(szTemp, "%4d %s", iLine, temp.c_str());
		source += szTemp;
	}

	return(source);
}

/* ************************************************************************* */
/*    Set Initialize Play Source  :  EDPLAY̏R}h̐ݒ            */
/* ************************************************************************* */

string Appli :: SetInitPlay(int iLine, LPTSTR pszInit)
{
	string		old;

	old = _init[iLine];
	_init[iLine] = pszInit;
	return(old);
}

/* ************************************************************************* */
/*    Get Initialize Play Source  :  EDPLAY̏R}h̎擾            */
/* ************************************************************************* */

inline string Appli :: GetInitPlay(int iLine)
{
	return(_init[iLine]);
}

/* ************************************************************************* */
/*    Set Initialize Play Line  :  EDPLAY̏R}h̍s̐ݒ        */
/* ************************************************************************* */

int Appli :: SetInitLine(int iLine)
{
	int		iLineOld;

	iLineOld = _iInitLine;
	_iInitLine = iLine;
	return(iLineOld);
}

/* ************************************************************************* */
/*    Get Initialize Play Line  :  EDPLAY̏R}h̍s̎擾        */
/* ************************************************************************* */

inline int Appli :: GetInitLine(void)
{
	return(_iInitLine);
}

