/* ***************************************** */
/*    Application Resource Class ver.1.12    */
/*                            'appres.cpp'   */
/*                          by T.Nakayama    */
/* ***************************************** */

#include "appres.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

AppRes :: AppRes()
 :	_hIcon(0), _hIconSm(0), _hCursor(0), _hbrBackground(0), _hAccel(0),
	_szMenu(NULL)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

AppRes :: ~AppRes()
{
	FreeMenuName();
}

/* ************************************************************************* */
/*    Set Icon From Resource                                                 */
/* ************************************************************************* */

HICON AppRes :: SetIconRes(HINSTANCE hInst, LPCTSTR szName)
{
	HICON	hHandle;

	hHandle = LoadIcon(hInst, szName);
	if(hHandle)
		hHandle = SetIcon(hHandle);
	return(hHandle);
}

/* ************************************************************************* */
/*    Set Icon                                                               */
/* ************************************************************************* */

HICON AppRes :: SetIcon(HICON hHandle)
{
	HICON	hIconOld;

	hIconOld = _hIcon;
	_hIcon = hHandle;
	return(hIconOld);
}

/* ************************************************************************* */
/*    Get Icon                                                               */
/* ************************************************************************* */

inline HICON AppRes :: GetIcon(void)
{
	return(_hIcon);
}

/* ************************************************************************* */
/*    Destory Icon                                                           */
/* ************************************************************************* */

inline BOOL AppRes :: DestroyIcon(void)
{
	return(:: DestroyIcon(_hIcon));
}

/* ************************************************************************* */
/*    Set Small Icon From Resource                                           */
/* ************************************************************************* */

HICON AppRes :: SetSmIconRes(HINSTANCE hInst, LPCTSTR szName)
{
	HICON	hHandle;

	hHandle = (HICON)LoadImage(hInst, szName, IMAGE_ICON, 16, 16,
															LR_DEFAULTCOLOR);
	if(hHandle)
		hHandle = SetSmIcon(hHandle);
	return(hHandle);
}

/* ************************************************************************* */
/*    Set Small Icon                                                         */
/* ************************************************************************* */

HICON AppRes :: SetSmIcon(HICON hHandle)
{
	HICON	hIconSmOld;

	hIconSmOld = _hIconSm;
	_hIconSm = hHandle;
	return(hIconSmOld);
}

/* ************************************************************************* */
/*    Get Small Icon                                                         */
/* ************************************************************************* */

inline HICON AppRes :: GetSmIcon(void)
{
	return(_hIconSm);
}

/* ************************************************************************* */
/*    Destory Small Icon                                                     */
/* ************************************************************************* */

inline BOOL AppRes :: DestroySmIcon(void)
{
	return(:: DestroyIcon(_hIconSm));
}

/* ************************************************************************* */
/*    Set Cursor From Resource                                               */
/* ************************************************************************* */

HCURSOR AppRes :: SetCursorRes(HINSTANCE hInst, LPCTSTR szName)
{
	HICON	hHandle;

	hHandle = LoadCursor(hInst, szName);
	if(hHandle)
		hHandle = SetCursor(hHandle);
	return(hHandle);
}

/* ************************************************************************* */
/*    Set Cursor                                                             */
/* ************************************************************************* */

HCURSOR AppRes :: SetCursor(HCURSOR hHandle)
{
	HICON	hCursorOld;

	hCursorOld = _hCursor;
	_hCursor = hHandle;
	return(hCursorOld);
}

/* ************************************************************************* */
/*    Get Cursor                                                             */
/* ************************************************************************* */

inline HCURSOR AppRes :: GetCursor(void)
{
	return(_hCursor);
}

/* ************************************************************************* */
/*    Destory Cursor                                                         */
/* ************************************************************************* */

inline BOOL AppRes :: DestroyCursor(void)
{
	return(:: DestroyCursor(_hCursor));
}

/* ************************************************************************* */
/*    Set Background Brush From Stock Object                                 */
/* ************************************************************************* */

HBRUSH AppRes :: SetBrushRes(int nBrush)
{
	HBRUSH	hHandle;

	hHandle = (HBRUSH)GetStockObject(nBrush);
	if(hHandle)
		hHandle = SetBrush(hHandle);
	return(hHandle);
}

/* ************************************************************************* */
/*    Set Background Brush                                                   */
/* ************************************************************************* */

HBRUSH AppRes :: SetBrush(HBRUSH hHandle)
{
	HBRUSH	hBrushOld;

	hBrushOld = _hbrBackground;
	_hbrBackground = hHandle;
	return(hBrushOld);
}

/* ************************************************************************* */
/*    Get Background Brush                                                   */
/* ************************************************************************* */

inline HBRUSH AppRes :: GetBrush(void)
{
	return(_hbrBackground);
}

/* ************************************************************************* */
/*    Set Menu Name                                                          */
/* ************************************************************************* */

int AppRes :: SetMenuName(LPCTSTR szBuf)
{
	int	iLength;

	FreeMenuName();
	iLength = strlen(szBuf);
	_szMenu = new char[iLength + 1];
	lstrcpy(_szMenu, szBuf);
	return(iLength);
}

/* ************************************************************************* */
/*    Get Menu Name                                                          */
/* ************************************************************************* */

inline LPCTSTR AppRes :: GetMenuName(void)
{
	return(_szMenu);
}

/* ************************************************************************* */
/*    Free Menu Name                                                         */
/* ************************************************************************* */

LPCTSTR AppRes :: FreeMenuName(void)
{
	_DELETEARRAY(_szMenu);
	return(_szMenu);
}

/* ************************************************************************* */
/*    Set Keyboard Accelerators From Resource                                */
/* ************************************************************************* */

HACCEL AppRes :: SetAcceleratorsRes(HINSTANCE hInst, LPCTSTR szBuf)
{
	HACCEL	hHandle;

	if(szBuf)
		hHandle = LoadAccelerators(hInst, szBuf);
	else
		hHandle = LoadAccelerators(hInst, GetMenuName());
	if(hHandle)
		hHandle = SetAccelerators(hHandle);
	return(hHandle);
}

/* ************************************************************************* */
/*    Set Keyboard Accelerators                                              */
/* ************************************************************************* */

HACCEL AppRes :: SetAccelerators(HACCEL hHandle)
{
	HACCEL	hAccelOld;

	hAccelOld = _hAccel;
	_hAccel = hHandle;
	return(hAccelOld);
}

/* ************************************************************************* */
/*    Get Keyboard Accelerators                                              */
/* ************************************************************************* */

inline HACCEL AppRes :: GetAccelerators(void)
{
	return(_hAccel);
}

