/* ************************************* */
/*    Base Application Class ver.1.11    */
/*                       'baseapp.cpp'   */
/*                      by T.Nakayama    */
/* ************************************* */

#include "baseapp.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

BaseAppli :: BaseAppli()
 :	_iWindowCount(0), _iProcTableCount(0), _pProcTable(NULL)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

BaseAppli :: ~BaseAppli()
{
	DestroyProcPairsTable();
}

/* ************************************************************************* */
/*    Set Procesure Pairs Table Count                                        */
/* ************************************************************************* */

int BaseAppli :: SetProcPairsTableCount(int iProcTableCount)
{
	int		iProcTableCountOld;

	iProcTableCountOld = _iProcTableCount;
	_iProcTableCount = iProcTableCount;
	return(iProcTableCountOld);
}

/* ************************************************************************* */
/*    Get Procesure Pairs Table Count                                        */
/* ************************************************************************* */

inline int BaseAppli :: GetProcPairsTableCount(void)
{
	return(_iProcTableCount);
}

/* ************************************************************************* */
/*    Clear Procesure Pairs Table Count                                      */
/* ************************************************************************* */

inline int BaseAppli :: ClearProcPairsTableCount(void)
{
	return(SetProcPairsTableCount(0));
}

/* ************************************************************************* */
/*    Set Window Count                                                       */
/* ************************************************************************* */

int BaseAppli :: SetWindowCount(int iWindowCount)
{
	int		iWindowCountOld;

	iWindowCountOld = _iWindowCount;
	_iWindowCount = iWindowCount;
	return(iWindowCountOld);
}

/* ************************************************************************* */
/*    Get Window Count                                                       */
/* ************************************************************************* */

inline int BaseAppli :: GetWindowCount(void)
{
	return(_iWindowCount);
}

/* ************************************************************************* */
/*    Increment Window Count                                                 */
/* ************************************************************************* */

inline int BaseAppli :: IncrementWindowCount(void)
{
	return(_iWindowCount++);
}

/* ************************************************************************* */
/*    Decrement Window Count                                                 */
/* ************************************************************************* */

inline int BaseAppli :: DecrementWindowCount(void)
{
	return(--_iWindowCount);
}

/* ************************************************************************* */
/*    Create Procesure Pairs Table                                           */
/* ************************************************************************* */

LPPROCPAIRS* BaseAppli :: CreateProcPairsTable(int iTables)
{
	SetProcPairsTableCount(iTables);
	_pProcTable = new LPPROCPAIRS[iTables];
	RemoveAllProcPairs();
	return(_pProcTable);
}

/* ************************************************************************* */
/*    Destroy Procesure Pairs Table                                          */
/* ************************************************************************* */

int BaseAppli :: DestroyProcPairsTable(void)
{
	DeleteAllProcPairs();
	_DELETEARRAY(_pProcTable);
	ClearProcPairsTableCount();
	return(BASEAPPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Resister Procedure Pairs                                               */
/* ************************************************************************* */

int BaseAppli :: RegisterProcPairs(LPPROCPAIRS pProcPairs)
{
	int				iTable;
	LPPROCPAIRS		pProcPairsNew;

	pProcPairsNew = new PROCPAIRS;
	*pProcPairsNew = *pProcPairs;
	iTable = FindProcPairs(NULL);
	if(iTable < 0)
		iTable = AddProcPairs(pProcPairsNew);
	else
		SetProcPairs(iTable, pProcPairsNew);
	IncrementWindowCount();
	return(iTable);
}

/* ************************************************************************* */
/*    Add Procedure Pairs                                                    */
/* ************************************************************************* */

int BaseAppli :: AddProcPairs(LPPROCPAIRS pProcPairs)
{
	int				i, iAdd, iTables;
	LPPROCPAIRS		*pProcTableOld;

	iAdd = GetProcPairsTableCount();
	iTables = iAdd + 1;
	pProcTableOld = _pProcTable;
	CreateProcPairsTable(iTables);
	for(i=0; i<iAdd; i++)
		SetProcPairs(i, pProcTableOld[i]);
	SetProcPairs(iAdd, pProcPairs);
	_DELETEARRAY(pProcTableOld);
	return(iAdd);
}

/* ************************************************************************* */
/*    Delete Procedure Pairs                                                 */
/* ************************************************************************* */

LPPROCPAIRS BaseAppli :: DeleteProcPairs(int iTable)
{
	LPPROCPAIRS	pProcPairs;

	pProcPairs = GetProcPairs(iTable);
	if(pProcPairs){
		pProcPairs = DestroyProcPairs(RemoveProcPairs(iTable));
		if(!pProcPairs)
			DecrementWindowCount();
	}
	return(pProcPairs);
}

/* ************************************************************************* */
/*    Delete All Procedure Pairs                                             */
/* ************************************************************************* */

int BaseAppli :: DeleteAllProcPairs(void)
{
	int		i;

	for(i=0; i<GetProcPairsTableCount(); i++)
		DeleteProcPairs(i);
	return(BASEAPPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Remove Procedure Pairs                                                 */
/* ************************************************************************* */

inline LPPROCPAIRS BaseAppli :: RemoveProcPairs(int iTable)
{
	return(SetProcPairs(iTable, NULL));
}

/* ************************************************************************* */
/*    Remove All Procedure Pairs                                             */
/* ************************************************************************* */

int BaseAppli :: RemoveAllProcPairs(void)
{
	int		i;

	for(i=0; i<GetProcPairsTableCount(); i++)
		RemoveProcPairs(i);
	return(BASEAPPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Destroy Procedure Pairs                                                */
/* ************************************************************************* */

LPPROCPAIRS BaseAppli :: DestroyProcPairs(LPPROCPAIRS pProcPairs)
{
	_DELETE(pProcPairs->pProc1);
	_DELETE(pProcPairs);
	return(pProcPairs);
}

/* ************************************************************************* */
/*    Set Procedure Pairs                                                    */
/* ************************************************************************* */

LPPROCPAIRS BaseAppli :: SetProcPairs(int iTable, LPPROCPAIRS pProcPairs)
{
	LPPROCPAIRS pProcPairsOld;

	pProcPairsOld = _pProcTable[iTable];
	_pProcTable[iTable] = pProcPairs;
	return(pProcPairsOld);
}

/* ************************************************************************* */
/*    Get Procedure Pairs                                                    */
/* ************************************************************************* */

inline LPPROCPAIRS BaseAppli :: GetProcPairs(int iTable)
{
	return(_pProcTable[iTable]);
}

/* ************************************************************************* */
/*    Get Procedure 1                                                        */
/* ************************************************************************* */

inline LPBaseProc BaseAppli :: GetProc1(int iTable)
{
	return(GetProcPairs(iTable) ? _pProcTable[iTable]->pProc1 : NULL);
}

/* ************************************************************************* */
/*    Get Procedure 2                                                        */
/* ************************************************************************* */

inline LPBaseProc BaseAppli :: GetProc2(int iTable)
{
	return(GetProcPairs(iTable) ? _pProcTable[iTable]->pProc2 : NULL);
}

/* ************************************************************************* */
/*    Get Window Handle                                                      */
/* ************************************************************************* */

inline HWND BaseAppli :: GetWindowHandle(int iTable)
{
	return(GetProcPairs(iTable) ? _pProcTable[iTable]->hWnd : 0);
}

/* ************************************************************************* */
/*    Find Procedure Pairs in Procedure Pairs Table                          */
/* ************************************************************************* */

int BaseAppli :: FindProcPairs(LPPROCPAIRS pProcPairs)
{
	int		i, iTable = -1;

	for(i=0; i<_iProcTableCount; i++){
		if(GetProcPairs(i) == pProcPairs){
			iTable = i;
			break;
		}
	}
	return(iTable);
}

/* ************************************************************************* */
/*    Find Procedure 1 in Procedure Pairs Table                              */
/* ************************************************************************* */

int BaseAppli :: FindProc1(LPBaseProc pProc)
{
	int		i, iTable = -1;

	for(i=0; i<_iProcTableCount; i++){
		if(GetProcPairs(i)){
			if(GetProc1(i) == pProc){
				iTable = i;
				break;
			}
		}
	}
	return(iTable);
}

/* ************************************************************************* */
/*    Find Procedure 2 in Procedure Pairs Table                              */
/* ************************************************************************* */

int BaseAppli :: FindProc2(LPBaseProc pProc)
{
	int		i, iTable = -1;

	for(i=0; i<_iProcTableCount; i++){
		if(GetProcPairs(i)){
			if(GetProc2(i) == pProc){
				iTable = i;
				break;
			}
		}
	}
	return(iTable);
}

/* ************************************************************************* */
/*    Find Window Handle in Procedure Pairs Table                            */
/* ************************************************************************* */

int BaseAppli :: FindWindowHandle(HWND hWnd)
{
	int		i, iTable = -1;

	for(i=0; i<_iProcTableCount; i++){
		if(GetProcPairs(i)){
			if(GetWindowHandle(i) == hWnd){
				iTable = i;
				break;
			}
		}
	}
	return(iTable);
}

/* ************************************************************************* */
/*    Send Message To All Windows                                            */
/* ************************************************************************* */

int BaseAppli :: SendMessageAllWindows(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int		i, iWindows = 0, iTables;
	HWND	hWnd;

	iWindows = GetWindowCount();
	iTables = GetProcPairsTableCount();
	for(i=0; i<iTables; i++){
		if(hWnd = GetWindowHandle(i)){
			SendMessage(hWnd, uMsg, wParam, lParam);
			iWindows--;
		}
	}
	return(iWindows);
}

