/* ********************************* */
/*    Command Line Class ver.1.00    */
/*                   'cmdline.cpp'   */
/*                  by T.Nakayama    */
/* ********************************* */

#include "cmdline.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

CmdLine :: CmdLine()
 :	_iParam(0), _pszParam(NULL)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

CmdLine :: ~CmdLine()
{
	FreeParam();
}

/* ************************************************************************* */
/*    Set Command Line                                                       */
/* ************************************************************************* */

int CmdLine :: SetCmdLine(LPCTSTR pszCmdLine)
{
	int			i, iLength;
	BOOL		bQuotation = FALSE;
	LPTSTR		pszTemp, p;

	FreeParam();
	iLength = strlen(pszCmdLine);
	if(!iLength)
		return(_iParam);
	pszTemp = p = new TCHAR[iLength+1];
	strcpy(pszTemp, pszCmdLine);

	// ======== Pass 1 
	while(*p == 0x20 || *p == 0x09) p++;
	while(*p != 0x00){
		_iParam++;
		while(*p != 0x00 && ((*p != 0x20 && *p != 0x09) || bQuotation)){
			if(*p == 0x22)
				bQuotation = ~bQuotation;
			p++;
		}
		*p = 0x00;
		while(*p == 0x20 || *p == 0x09) p++;
	}

	// ======== Pass 2 
	p = pszTemp;
	_pszParam = new LPTSTR[_iParam];
	for(i=0; i<_iParam; i++){
		while(*p == 0x20 || *p == 0x09) p++;
		iLength = strlen(p);
		_pszParam[i] = new TCHAR[iLength+1];
		strcpy(_pszParam[i], p);
		p += iLength;
	}

	_DELETE(pszTemp);
	return(_iParam);
}

/* ************************************************************************* */
/*    Get Parameter Count                                                    */
/* ************************************************************************* */

inline int CmdLine :: GetParamCount(void)
{
	return(_iParam);
}

/* ************************************************************************* */
/*    Get Parameter                                                          */
/* ************************************************************************* */

inline LPCTSTR CmdLine :: GetParam(int iParam)
{
	return((LPCTSTR)_pszParam[iParam]);
}

/* ************************************************************************* */
/*    Free Parameter                                                         */
/* ************************************************************************* */

int CmdLine :: FreeParam(void)
{
	int		i;

	for(i=0; i<_iParam; i++)
		_DELETEARRAY(_pszParam[i]);
	_DELETEARRAY(_pszParam);
	_iParam = 0;

	return(CMDLINE_COMPLETE);
}

/* ************************************************************************* */
/*    Remove Quotation                                                       */
/* ************************************************************************* */

int CmdLine :: RemoveQuotation(LPTSTR pszCmdLine)
{
	int			i, iLength;
	LPTSTR		p1, p2;

	iLength = strlen(pszCmdLine);
	p1 = p2 = pszCmdLine;
	for(i=0; i<iLength; i++, p1++){
		if(*p1 != 0x22)
			*p2++ = *p1;
	}
	*p2 = 0x00;
	return(strlen(pszCmdLine));
}

/* ************************************************************************* */
/*    Remove Quotation in Parameter                                          */
/* ************************************************************************* */

inline int CmdLine :: RemoveParamQuotation(int iParam)
{
	return(RemoveQuotation(_pszParam[iParam]));
}
