/* **************************************** */
/*    Dialogbox Procedure Class ver.1.11    */
/*                          'dlgproc.cpp'   */
/*                         by T.Nakayama    */
/* **************************************** */

#include "dlgproc.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

DlgProc :: DlgProc()
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

DlgProc :: ~DlgProc()
{
}

/* ************************************************************************* */
/*    Centering Dialogbox                                                    */
/* ************************************************************************* */

int DlgProc :: CenteringDialog(HWND hDlg)
{
	RECT	rcWnd, rcDlg;

	GetWindowRect(GetParent(hDlg), &rcWnd);
	GetWindowRect(hDlg, &rcDlg);
	OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top);
	MoveWindow(hDlg,
		rcWnd.left+((rcWnd.right-rcWnd.left)-rcDlg.right)/2,
		rcWnd.top+((rcWnd.bottom-rcWnd.top)-rcDlg.bottom)/2,
		rcDlg.right, rcDlg.bottom, FALSE);
	return(DIALOG_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND                                                             */
/* ************************************************************************* */

int DlgProc :: Command(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam)){
		case IDOK:		// ------------------------------------------ IDOK ----
			EndDialog(hDlg, TRUE);
			return(DIALOG_COMPLETE);

		case IDCANCEL:	// -------------------------------------- IDCANCEL ----
			EndDialog(hDlg, FALSE);
			return(DIALOG_COMPLETE);
	}
	return(DIALOG_NO_PROCESS);
}

/* ************************************************************************* */
/*    WM_PAINT                                                               */
/* ************************************************************************* */

int DlgProc :: Paint(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	return(DIALOG_NO_PROCESS);
}

/* ************************************************************************* */
/*    WM_NOTIFY                                                              */
/* ************************************************************************* */

int DlgProc :: Notify(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	return(DIALOG_NO_PROCESS);
}

/* ************************************************************************* */
/*    WM_INITDIALOG                                                          */
/* ************************************************************************* */

int DlgProc :: InitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	int		iResult;

	// ======== Get a parent window handle
	_hInst = (HINSTANCE)GetWindowLong(GetParent(hDlg), GWL_HINSTANCE);

	// ======== WM_CREATE Appendix ========
	iResult = InitDialogAppendix(hDlg, wParam, lParam);

	return(iResult);
}

/* ************************************************************************* */
/*    WM_INITDIALOG Appendix                                                 */
/* ************************************************************************* */

int DlgProc :: InitDialogAppendix(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	CenteringDialog(hDlg);

	return(DIALOG_COMPLETE);
}

/* ************************************************************************* */
/*    Dialog Procedure                                                       */
/* ************************************************************************* */

BOOL DlgProc :: DlgProcedure(HWND hDlg, UINT uMsg, WPARAM wParam,LPARAM lParam)
{
	switch(uMsg){
		case WM_COMMAND:	// ++++++++++++++++++++++++++++++++ WM_COMMAND ++++
			if(Command(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_NOTIFY:	// +++++++++++++++++++++++++++++++++++++ WM_NOTIFY ++++
			if(Notify(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_PAINT:	// ++++++++++++++++++++++++++++++++++++++ WM_PAINT ++++
			if(Paint(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

		case WM_INITDIALOG:	// +++++++++++++++++++++++++++++ WM_INITDIALOG ++++
			if(InitDialog(hDlg, wParam, lParam) == DIALOG_COMPLETE)
				return(TRUE);
			break;

	}
	return(FALSE);
}

