/* ******************************************* */
/*    EDPLAY List Translator Class ver.1.00    */
/*                             'edtrans.cpp'   */
/*                            by T.Nakayama    */
/* ******************************************* */

#include "edtrans.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

EdTrans :: EdTrans()
 :	_iChannel(MMLCHANNEL_NOUSE), _iOctave(4), _iLength(MMLLEN_8), _iVolume(12)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

EdTrans :: ~EdTrans()
{
}

/* ************************************************************************* */
/*    Set Channel                                                            */
/* ************************************************************************* */

int EdTrans :: SetChannel(int iChannel)
{
	int		iChannelOld;

	iChannelOld = _iChannel;
	_iChannel = iChannel;
	return(iChannelOld);
}

/* ************************************************************************* */
/*    Get Channel                                                            */
/* ************************************************************************* */

inline int EdTrans :: GetChannel(void)
{
	return(_iChannel);
}

/* ************************************************************************* */
/*    Set Octave                                                             */
/* ************************************************************************* */

int EdTrans :: SetOctave(int iOctave)
{
	int		iOctaveOld;

	iOctaveOld = _iOctave;
	_iOctave = iOctave;
	return(iOctaveOld);
}

/* ************************************************************************* */
/*    Get Octave                                                             */
/* ************************************************************************* */

inline int EdTrans :: GetOctave(void)
{
	return(_iOctave);
}

/* ************************************************************************* */
/*    Set Length                                                             */
/* ************************************************************************* */

int EdTrans :: SetLength(int iLength)
{
	int		iLengthOld;

	iLengthOld = _iLength;
	_iLength = iLength;
	return(iLengthOld);
}

/* ************************************************************************* */
/*    Get Length                                                             */
/* ************************************************************************* */

inline int EdTrans :: GetLength(void)
{
	return(_iLength);
}

/* ************************************************************************* */
/*    Set Volume                                                             */
/* ************************************************************************* */

int EdTrans :: SetVolume(int iVolume)
{
	int		iVolumeOld;

	iVolumeOld = _iVolume;
	_iVolume = iVolume;
	return(iVolumeOld);
}

/* ************************************************************************* */
/*    Get Volume                                                             */
/* ************************************************************************* */

inline int EdTrans :: GetVolume(void)
{
	return(_iVolume);
}

/* ************************************************************************* */
/*    Get Code Text                                                          */
/* ************************************************************************* */

int EdTrans :: GetCodeText(LPTSTR pszText, int iCode)
{
	static TCHAR *szCode[MMLCODENUM_COUNT]\
	 = {"c", "c+", "d", "d+", "e", "f", "f+", "g", "g+", "a", "a+", "b", "r"};

	if(iCode >= 0 && iCode < MMLCODENUM_COUNT)
		strcpy(pszText, szCode[iCode]);
	else
		pszText[0] = 0x00;
	return(strlen(pszText));
}

/* ************************************************************************* */
/*    Get Length Text                                                        */
/* ************************************************************************* */

int EdTrans :: GetLengthText(LPTSTR pszText, int iLength)
{
	static TCHAR *szLength[MMLLEN_COUNT]\
		= {"1", "2.", "2", "4.", "4", "8.", "6", "8", "16.", "12", "16",
		"32.", "24", "32", "48", "64", "96", "192"};

	if(iLength >= 0 && iLength < MMLLEN_COUNT)
		strcpy(pszText, szLength[iLength]);
	else
		pszText[0] = 0x00;
	return(strlen(pszText));
}

/* ************************************************************************* */
/*    Initialize Iterator                                                    */
/* ************************************************************************* */

int EdTrans :: InitIterator(LPMmlChannel pMmlChannel)
{
	_ite = pMmlChannel->GetBegin();
	return(EDTRANS_COMPLETE);
}

inline int EdTrans :: InitIterator(LPMmlData pMmlData, int iChannel)
{
	return(InitIterator(pMmlData->GetMmlChannel(iChannel)));
}

/* ************************************************************************* */
/*    Translate                                                              */
/* ************************************************************************* */

int EdTrans :: Translate(LPMmlChannel pMmlChannel, string &mml, UINT uFlag)
{
	char		szTemp[STRING_BUFFER];
	string		buf;
	LPMmlEvent	pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	mml = "";
	if(uFlag & EDTRANS_ITEBEGIN)
		ite = pMmlChannel->GetBegin();
	else
		ite = _ite;

	if(uFlag & EDTRANS_SETOCTAVE){
		sprintf(szTemp, "o%d", GetOctave());
		mml += szTemp;
	}
	if(uFlag & EDTRANS_SETLENGTH){
		GetLengthText(szTemp, GetLength());
		mml += "l";
		mml += szTemp;
	}
//	if(uFlag & EDTRANS_SETVOLUME){
//		sprintf(szTemp, "v%d", GetVolume());
//		mml += szTemp;
//	}

	while(ite != pMmlChannel->GetEnd()){
		pEvent = *ite++;
		if(pEvent->GetCode() == MMLCODE_BAR)
			break;
		TranslateEvent(pEvent, buf);
		mml += buf;
	}
	_ite = ite;
	return(EDTRANS_COMPLETE);
}

inline int EdTrans :: Translate(
	LPMmlData pMmlData, int iChannel, string &mml, UINT uFlag)
{
	return(Translate(pMmlData->GetMmlChannel(iChannel), mml, uFlag));
}

/* ************************************************************************* */
/*    Translate Event                                                        */
/* ************************************************************************* */

int EdTrans :: TranslateEvent(LPMmlEvent pEvent, string &mml)
{
	int			i, iCode, iOctave, iLength, iDelta;
	TCHAR		szTemp[STRING_BUFFER];
	static int	iLengthPriod[MMLLEN_COUNT]
		= {	MMLLEN_NONE, MMLLEN_2, MMLLEN_NONE, MMLLEN_4, MMLLEN_NONE,
			MMLLEN_8, MMLLEN_NONE, MMLLEN_NONE, MMLLEN_16, MMLLEN_NONE,
			MMLLEN_NONE, MMLLEN_32, MMLLEN_NONE, MMLLEN_NONE, MMLLEN_NONE,
			MMLLEN_NONE, MMLLEN_NONE, MMLLEN_NONE};

	mml = "";
	iCode = (int)pEvent->GetCode();
	iLength = pEvent->GetLength(); 
	if(iCode == MMLCODE_BAR)
		return(EDTRANS_COMPLETE);

	if(iCode == MMLCODE_REST){
		iOctave = GetOctave();
		iCode = MMLCODENUM_REST;
	}
	else{
		iOctave = (int)(iCode / MMLCODE_COUNT) - 1;
		iCode = iCode % MMLCODE_COUNT;
	}

	// IN^[u̐ݒ
	if(iOctave != GetOctave()){
		iDelta = iOctave - GetOctave();
		if(iDelta > 0)
			for(i = 0; i < iDelta; i++)
				mml += ">";
		else
			for(i = 0; i < -iDelta; i++)
				mml += "<";
		SetOctave(iOctave);
	}

	// ̐ݒ
	GetCodeText(szTemp, iCode);
	mml += szTemp;

	// ̐ݒ
	if(iLength != GetLength()){
		if(iLengthPriod[iLength] == GetLength())
			mml += ".";
		else{
			GetLengthText(szTemp, iLength);
			mml += szTemp;
		}
	}

	// ^C̐ݒ
	if(pEvent->GetParam() & MMLPARAM_TAI)
		mml += "&";

	return(EDTRANS_COMPLETE);
}

