/* ************************************** */
/*    Global Function Library ver.1.10    */
/*                           'glblib.c'   */
/*                       by T.Nakayama    */
/* ************************************** */

#include "glblib.h"

/* ************************************************************************* */
/*    Search Character in String                                             */
/* ************************************************************************* */

int Instr(LPCTSTR pString, TCHAR c)
{
	int i = 0;

	for(; *pString; i++,pString++)
		if(*pString == c)
			return(i);
	return(-1);
}

/* ************************************************************************* */
/*    Integer to String With Comma                                           */
/* ************************************************************************* */

int IntToStrComma(int iValue, LPTSTR pszValue)
{
	int		i, n1, n2;
	TCHAR	szBuf[STRING_BUFFER];
	LPTSTR	p;

	wsprintf(szBuf, "%d", iValue);
	n1 = n2 = lstrlen(szBuf);
	p = pszValue;
	for(i = 0; i < n1; i++){
		*p++ = *(szBuf + i);
		n2--;
		if(((n2 % 3) == 0) && (n2 > 0)) *p++ = ',';
	}
	*p = 0x00;
	return(lstrlen(pszValue));
}

/* ************************************************************************* */
/*    String to Integer With Comma                                           */
/* ************************************************************************* */

int StrToIntComma(LPCTSTR pszValue)
{
	int		i, iLength, iValue = 0;
	TCHAR	c;

	iLength = strlen(pszValue);
	for(i = 0; i < iLength; i++){
		c = *(pszValue + i);
		if(c >= '0' && c <= '9')
			iValue = iValue * 10 + (int)(c - '0');
	}
	return(iValue);
}

/* ************************************************************************* */
/*    N String to Long                                                       */
/* ************************************************************************* */

LONG StrToLongN(LPCTSTR pString, int iLength)
{
	return(StrToLongNBase(pString, iLength, 0));
}

/* ************************************************************************* */
/*    N String to Long With Base Value                                       */
/* ************************************************************************* */

LONG StrToLongNBase(LPCTSTR pString, int iLength, int iBase)
{
	LONG	lValue;
	LPTSTR	pszBuf;

	pszBuf = (LPTSTR)calloc(iLength+1, sizeof(TCHAR));
	strncpy(pszBuf, pString, iLength);
	lValue = StrToLongBase(pString, iBase);
	free(pszBuf);
	return(lValue);
}

/* ************************************************************************* */
/*    String to Long                                                         */
/* ************************************************************************* */

LONG StrToLong(LPCTSTR pString)
{
	return(StrToLongBase(pString, 0));
}

/* ************************************************************************* */
/*    String to Long With Base Value                                         */
/* ************************************************************************* */

LONG StrToLongBase(LPCTSTR pString, int iBase)
{
	int		iHeader;
	LONG	lValue;
	LPSTR	pEndPtr;
	int		iBaseTable[] = {2, 10, 16, 10, 10};
	LPTSTR	szBaseTable = "%#$+-";

	iHeader = Instr(szBaseTable, pString[0]);
	if(0 <= iHeader && iHeader < sizeof(szBaseTable))
		iBase = iBaseTable[iHeader], pString++;
	lValue = strtol(pString, &pEndPtr, iBase);
	return(lValue);
}

/* ************************************************************************* */
/*    N String to Unsigned Long                                              */
/* ************************************************************************* */

ULONG StrToUlongN(LPCTSTR pString, int iLength)
{
	return(StrToUlongNBase(pString, iLength, 0));
}

/* ************************************************************************* */
/*    N String to Unsigned Long With Base Value                              */
/* ************************************************************************* */

ULONG StrToUlongNBase(LPCTSTR pString, int iLength, int iBase)
{
	ULONG	ulValue;
	LPTSTR	pszBuf;

	pszBuf = (LPTSTR)calloc(iLength+1, sizeof(TCHAR));
	strncpy(pszBuf, pString, iLength);
	ulValue = StrToUlongBase(pString, iBase);
	free(pszBuf);
	return(ulValue);
}

/* ************************************************************************* */
/*    String to Unsigned Long                                                */
/* ************************************************************************* */

ULONG StrToUlong(LPCTSTR pString)
{
	return(StrToUlongBase(pString, 0));
}

/* ************************************************************************* */
/*    String to Unsigned Long With Base Value                                */
/* ************************************************************************* */

ULONG StrToUlongBase(LPCTSTR pString, int iBase)
{
	int		iHeader;
	UINT	ulValue;
	LPSTR	pEndPtr;
	int		iBaseTable[] = {2, 10, 16, 10, 10};
	LPTSTR	szBaseTable = "%#$+-";

	iHeader = Instr(szBaseTable, pString[0]);
	if(0 <= iHeader && iHeader < sizeof(szBaseTable))
		iBase = iBaseTable[iHeader], pString++;
	ulValue = strtoul(pString, &pEndPtr, iBase);
	return(ulValue);
}

/* ************************************************************************* */
/*    Is Text File?                                                          */
/* ************************************************************************* */

BOOL IsTextFile(LPBYTE pFile, DWORD dwFileSize, LPCTSTR pszTextCode)
{
	DWORD	i = 0;
	LPBYTE	p;
	BOOL	bText = TRUE;

	if(dwFileSize == 0)
		return(FALSE);

	p = pFile;
	while(i < (dwFileSize - 1)){
		if(isspace(*p) || isprint(*p) || (0xa1<=*p && *p<=0xdf) || *p==0){
			p++, i++;
			continue;
		}
		if(pszTextCode){
			if(strchr(pszTextCode, *p)){
				p++, i++;
				continue;
			}
		}
		if((0x81<=*p && *p<=0x9f) || (0xe0<=*p && *p<=0xfc)){
			p++, i++;
			if((0x40<=*p && *p<=0x7e) || (0x80<=*p && *p<=0xfc)){
				p++, i++;
				continue;
			}
		}
		if(*p == 0x80){
			p++, i++;
			continue;
		}
		bText = FALSE;
		break;
	}
	return(bText);
}

/* ************************************************************************* */
/*    Replace Graphic Character Code                                         */
/* ************************************************************************* */

BOOL ReplaceGraphicChar(
	LPBYTE pFile, DWORD dwFileSize, LPCTSTR pszTextCode, TCHAR cCode)
{
	DWORD	i = 0;
	LPBYTE	p;
	BOOL	bText = TRUE;

	if(dwFileSize == 0)
		return(FALSE);

	p = pFile;
	while(i < (dwFileSize - 1)){
		if(isspace(*p) || isprint(*p) || (0xa1<=*p && *p<=0xdf) || *p==0){
			p++, i++;
			continue;
		}
		if(pszTextCode){
			if(strchr(pszTextCode, *p)){
				p++, i++;
				continue;
			}
		}
		if((0x81<=*p && *p<=0x9f) || (0xe0<=*p && *p<=0xfc)){
			p++, i++;
			if((0x40<=*p && *p<=0x7e) || (0x80<=*p && *p<=0xfc)){
				p++, i++;
			}
			else{
				*(p-1) = (BYTE)cCode;
			}
			continue;
		}
		if(*p == 0x80){
			*p = (BYTE)cCode;
			p++, i++;
			continue;
		}
		bText = FALSE;
		break;
	}
	return(bText);
}

