/* ****************************** */
/*    List View Class ver.1.20    */
/*               'listview.cpp'   */
/*               by T.Nakayama    */
/* ****************************** */

#include "listview.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

ListView :: ListView()
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

ListView :: ~ListView()
{
	if(ListView_GetImageList(_hWnd, LVSIL_NORMAL))
		ImageList_Destroy(ListView_GetImageList(_hWnd, LVSIL_NORMAL));

	if(ListView_GetImageList(_hWnd, LVSIL_SMALL))
		ImageList_Destroy(ListView_GetImageList(_hWnd, LVSIL_SMALL));
}

/* ************************************************************************* */
/*    Create Window                                                          */
/* ************************************************************************* */

HWND ListView :: CreateWindow_(void)
{
	_hWnd = CreateWindowEx(0, WC_LISTVIEW, "",
		WS_CHILD | WS_VISIBLE | LVS_REPORT, 0, 0, 0, 0,
		_hParentWnd, (HMENU)CONTROLID_LISTVIEW, _hInst, NULL);
	return(_hWnd);
}

/* ************************************************************************* */
/*    Create Image List                                                      */
/* ************************************************************************* */

int ListView :: CreateImageList(int iIcons, int cx, int cy, int iImageList, int iIconID)
{
	int			i;
	HIMAGELIST	hImage;

	// -------- Create image lists
	hImage = ImageList_Create(cx, cy, ILC_COLOR4 | ILC_MASK, iIcons, 1);
	if(!hImage)
		return(LISTVIEW_NOT_CREATE_IMAGELIST);

	// -------- Set image lists of large and small icon on list view
	ListView_SetImageList(_hWnd, hImage, iImageList);

	// -------- Additional icon to image lists
	for(i=0; i<iIcons; i++)
		ImageList_AddIcon(hImage, LoadIcon(_hInst,MAKEINTRESOURCE(iIconID+i)));

	return(LISTVIEW_COMPLETE);
}

/* ************************************************************************* */
/*    Get Item Count                                                         */
/* ************************************************************************* */

inline int ListView :: GetItemCount(void)
{
	return(ListView_GetItemCount(_hWnd));
}

/* ************************************************************************* */
/*    Insert Item                                                            */
/* ************************************************************************* */

inline int ListView :: InsertItem(LV_ITEM *pItem)
{
	return(ListView_InsertItem(_hWnd, pItem));
}

/* ************************************************************************* */
/*    Set Item                                                               */
/* ************************************************************************* */

inline int ListView :: SetItem(LV_ITEM *pItem)
{
	return(ListView_SetItem(_hWnd, pItem));
}

/* ************************************************************************* */
/*    Get Item                                                               */
/* ************************************************************************* */

inline int ListView :: GetItem(LV_ITEM *pItem)
{
	return(ListView_GetItem(_hWnd, pItem));
}

/* ************************************************************************* */
/*    Delete Item                                                            */
/* ************************************************************************* */

inline int ListView :: DeleteItem(int iItem)
{
	return(ListView_DeleteItem(_hWnd, iItem));
}

/* ************************************************************************* */
/*    Delete All Items                                                       */
/* ************************************************************************* */

inline int ListView :: DeleteAllItems(void)
{
	return(ListView_DeleteAllItems(_hWnd));
}

/* ************************************************************************* */
/*    Set Focused Item                                                       */
/* ************************************************************************* */

int ListView :: SetFocusedItem(int iItem)
{
	LV_ITEM	item;

	item.iItem = iItem;
	item.iSubItem = 0;
	item.mask = LVIF_STATE;
	item.state = LVIS_FOCUSED | LVIS_SELECTED;
	item.stateMask = LVIS_FOCUSED | LVIS_SELECTED;
	return((int)ListView_SetItem(_hWnd, &item));
}

/* ************************************************************************* */
/*    Get Focused Item                                                       */
/* ************************************************************************* */

int ListView :: GetFocusedItem(void)
{
	return(ListView_GetNextItem(_hWnd, -1, LVNI_FOCUSED | LVNI_ALL));
}

/* ************************************************************************* */
/*    Get lParam of Focused Item                                             */
/* ************************************************************************* */

inline LPARAM ListView :: GetFocusedItemParam(void)
{
	return(GetParam(GetFocusedItem()));
}

/* ************************************************************************* */
/*    Get Selected Count                                                     */
/* ************************************************************************* */

inline int ListView :: GetSelectedCount(void)
{
	return((int)ListView_GetSelectedCount(_hWnd));
}

/* ************************************************************************* */
/*    Get Next Item                                                          */
/* ************************************************************************* */

inline int ListView :: GetNextItem(int iStart, UINT uFlags)
{
	return((int)ListView_GetNextItem(_hWnd, iStart, uFlags));
}

/* ************************************************************************* */
/*    Set lParam                                                             */
/* ************************************************************************* */

int ListView :: SetParam(int iItem, LPARAM lParam)
{
	LV_ITEM		item;

	item.iItem		= iItem;
	item.mask		= LVIF_PARAM;
	item.iSubItem	= 0;
	item.lParam		= lParam;
	return(SetItem(&item));
}

/* ************************************************************************* */
/*    Get lParam                                                             */
/* ************************************************************************* */

LPARAM ListView :: GetParam(int iItem)
{
	LV_ITEM			item;

	item.iItem		= iItem;
	item.mask		= LVIF_PARAM;
	item.iSubItem	= 0;
	GetItem(&item);
	return(item.lParam);
}

/* ************************************************************************* */
/*    Find Item                                                              */
/* ************************************************************************* */

inline int ListView :: FindItem(LV_FINDINFO *pFindInfo)
{
	return(ListView_FindItem(_hWnd, -1, pFindInfo));
}

/* ************************************************************************* */
/*    Find Item with Param                                                   */
/* ************************************************************************* */

int ListView :: FindItemParam(LPARAM lParam)
{
	LV_FINDINFO		findinfo;

	findinfo.flags	= LVFI_PARAM;
	findinfo.lParam	= lParam;
	return(FindItem(&findinfo));
}

/* ************************************************************************* */
/*    Clear State                                                            */
/* ************************************************************************* */

int ListView :: ClearState(int iItem)
{
	LV_ITEM	item;

	item.iItem = iItem;
	item.iSubItem = 0;
	item.mask = LVIF_STATE;
	item.state = 0;
	item.stateMask = (UINT)-1;
	return((int)ListView_SetItem(_hWnd, &item));
}

/* ************************************************************************* */
/*    Clear Item                                                             */
/* ************************************************************************* */

int ListView :: ClearItem(int iItem, int iSubItem)
{
	LV_ITEM	item;
	TCHAR	szBuf[1];

	szBuf[0] = 0x00;
	item.iItem = iItem;
	item.iSubItem = iItem;
	item.mask = LVIF_TEXT;
	item.pszText = szBuf;
	return(SetItem(&item));
}

/* ************************************************************************* */
/*    Load Column                                                            */
/* ************************************************************************* */

int ListView :: LoadColumn(LPCTSTR pszRcDataID)
{
	int			i, iColumns = 0;
	LPBYTE		pData;
	TCHAR		szText[STRING_BUFFER];
	HANDLE		hColumn;
	RC_COLUMN	*pColumn;
	LV_COLUMN	col;

	// -------- Load Column Data
	hColumn = LoadResource(_hInst,FindResource(_hInst,pszRcDataID, RT_RCDATA));
	pData = (LPBYTE)LockResource(hColumn);
	iColumns = (int)*((WORD*)pData);
	pColumn = (RC_COLUMN*)(pData+sizeof(WORD));

	// -------- Set Column
	col.mask	= LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	col.pszText	= szText;
	for(i=0; i<iColumns; i++){
		LoadString(_hInst, pColumn[i].wTextID, szText, STRING_BUFFER);
		col.iSubItem	= i;
		col.cx			= (int)pColumn[i].cx;
		col.fmt			= (int)pColumn[i].fmt;
		InsertColumn(i, &col);
	}
	UnlockResource(hColumn);
	FreeResource(hColumn);

	return(iColumns);
}

/* ************************************************************************* */
/*    Insert Column                                                          */
/* ************************************************************************* */

inline int ListView :: InsertColumn(int iCol, LV_COLUMN *pColumn)
{
	return(ListView_InsertColumn(_hWnd, iCol, pColumn));
}

/* ************************************************************************* */
/*    Set Column                                                             */
/* ************************************************************************* */

inline BOOL ListView :: SetColumn(int iCol, LV_COLUMN *pColumn)
{
	return((BOOL)ListView_SetColumn(_hWnd, iCol, pColumn));
}

/* ************************************************************************* */
/*    Get Column                                                             */
/* ************************************************************************* */

inline BOOL ListView :: GetColumn(int iCol, LV_COLUMN *pColumn)
{
	return((BOOL)ListView_GetColumn(_hWnd, iCol, pColumn));
}

/* ************************************************************************* */
/*    Get Number of Columns                                                  */
/* ************************************************************************* */

int ListView :: GetNumColumns(void)
{
	int		iNum = 0;

	while(IsColumnExists(iNum)) iNum++;

	return(iNum);
}

/* ************************************************************************* */
/*    Is Column Exits?                                                       */
/* ************************************************************************* */

BOOL ListView :: IsColumnExists(int iCol)
{
	LV_COLUMN	lvc;

	lvc.mask = LVCF_WIDTH;

	return(GetColumn(iCol, &lvc));
}

/* ************************************************************************* */
/*    Ensure Visible                                                         */
/* ************************************************************************* */

inline BOOL ListView :: EnsureVisible(int iItem, BOOL bPartialOK)
{
	return(ListView_EnsureVisible(_hWnd, iItem, bPartialOK));
}

/* ************************************************************************* */
/*    Hit Test                                                               */
/* ************************************************************************* */

inline int ListView :: HitTest(LV_HITTESTINFO *pInfo)
{
	return(ListView_HitTest(_hWnd, pInfo));
}

