/* ********************************************* */
/*    MIDI to EDPLAY Procedure Class ver.1.00    */
/*                               'm2eproc.cpp'   */
/*                              by T.Nakayama    */
/* ********************************************* */

#include "m2eproc.h"
#include <commctrl.h> 

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

Midi2EdProc :: Midi2EdProc()
 :	_pSmfData(NULL), _pMmlData(NULL), _uStatus(STATUS_OPENFILE),
	_pszFileName(NULL), _hFocus(0)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

Midi2EdProc :: ~Midi2EdProc()
{
	FreeFileName();
	_appli.FreeMmlData(_pMmlData);
	_appli.FreeMidiData(_pSmfData);
}

/* ************************************************************************* */
/*    Centering Window                                                       */
/* ************************************************************************* */

int Midi2EdProc :: CenteringWindow(HWND hWnd)
{
	RECT	rc;

	GetWindowRect(hWnd, &rc);
	OffsetRect(&rc, -rc.left, -rc.top);
	MoveWindow(hWnd,
		((GetSystemMetrics(SM_CXSCREEN)-rc.right)/2+4)&(~7),
		(GetSystemMetrics(SM_CYSCREEN)-rc.bottom)/2,
		rc.right, rc.bottom, FALSE);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Initialize Combobox                                                    */
/* ************************************************************************* */

int Midi2EdProc :: InitCombobox(HWND hWnd, int iItemCount, UINT uTextID)
{
	int		i, iIndex;
	TCHAR	szText[STRING_BUFFER];

	for(i = 0; i<iItemCount; i++){
		LoadString(GetInstance(), uTextID + i, szText, sizeof(szText));
		iIndex = SendMessage(hWnd, CB_ADDSTRING, 0, (LPARAM)szText);
		SendMessage(hWnd, CB_SETITEMDATA, iIndex, i);
	}
	return(i);
}

/* ************************************************************************* */
/*    Initialize MIDI Information                                            */
/* ************************************************************************* */

int Midi2EdProc :: InitMidiInfo(LPMIDIINFO lpmidiinfo)
{
	memset(lpmidiinfo->szFileName, 0, STRING_BUFFER);
	lpmidiinfo->iFormat = MIDIINFO_FORMAT;
	lpmidiinfo->iTimeBase = MIDIINFO_TIMEBASE;
	lpmidiinfo->iChannelCount = MIDIINFO_CHANNEL;
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Initialize Play Information                                            */
/* ************************************************************************* */

int Midi2EdProc :: InitPlayInfo(LPPLAYINFO lpplayinfo)
{
	lpplayinfo->iPartCount = PLAYINFO_PART;
	lpplayinfo->iResolution = PLAYINFO_RESOLUTION;
	lpplayinfo->iDivision = PLAYINFO_DIVISION;
	lpplayinfo->iLineNumber = PLAYINFO_LINENUMBER;
	lpplayinfo->iPlayType = PLAYINFO_PLAYTYPE;
	memset(lpplayinfo->szFileName, 0, STRING_BUFFER);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Initialize MIDI Channel List                                           */
/* ************************************************************************* */

inline int Midi2EdProc :: InitMidiChList(LPListView lpmidilist)
{
	return(lpmidilist->DeleteAllItems());
}

/* ************************************************************************* */
/*    Initialize Play Channel List                                           */
/* ************************************************************************* */

inline int Midi2EdProc :: InitPlayChList(LPListView lpplaylist)
{
	return(lpplaylist->DeleteAllItems());
}

/* ************************************************************************* */
/*    Initialize Source View                                                 */
/* ************************************************************************* */

inline string Midi2EdProc :: InitSourceView(string &source)
{
	return(source = "");
}

/* ************************************************************************* */
/*    Set MIDI Information                                                   */
/* ************************************************************************* */

int Midi2EdProc :: SetMidiInfo(HWND hWnd, LPMIDIINFO lpmidiinfo)
{
	TCHAR	szTemp[STRING_BUFFER];

	SetWindowText(GetDlgItem(hWnd, IDC_FILENAME), lpmidiinfo->szFileName);
	wsprintf(szTemp, "%1d",lpmidiinfo->iFormat);
	SetWindowText(GetDlgItem(hWnd, IDC_FORMAT), szTemp);
	IntToStrComma(lpmidiinfo->iTimeBase, szTemp);
	SetWindowText(GetDlgItem(hWnd, IDC_TIMEBASE), szTemp);
	wsprintf(szTemp, "%d",lpmidiinfo->iChannelCount);
	SetWindowText(GetDlgItem(hWnd, IDC_MIDICHANNEL), szTemp);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Set Play Information                                                   */
/* ************************************************************************* */

int Midi2EdProc :: SetPlayInfo(HWND hWnd, LPPLAYINFO lpplayinfo)
{
	TCHAR	szTemp[STRING_BUFFER];

	wsprintf(szTemp, "%d",lpplayinfo->iPartCount);
	SetWindowText(GetDlgItem(hWnd, IDC_EDPLAYCHANNEL), szTemp);
	SendMessage(GetDlgItem(hWnd, IDC_RESOLUTION), CB_SETCURSEL,
		lpplayinfo->iResolution, 0);
	IntToStrComma(lpplayinfo->iDivision, szTemp);
	SetWindowText(GetDlgItem(hWnd, IDC_DIVISION), szTemp);
	IntToStrComma(lpplayinfo->iLineNumber, szTemp);
	SetWindowText(GetDlgItem(hWnd, IDC_LINENUMBER), szTemp);
	SendMessage(GetDlgItem(hWnd, IDC_PLAYTYPE), CB_SETCURSEL,
		lpplayinfo->iPlayType, 0);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Set Source View                                                        */
/* ************************************************************************* */

int Midi2EdProc :: SetSourceView(HWND hWnd, string &source)
{
	return(SetWindowText(GetDlgItem(hWnd, IDC_SOURCE), source.c_str()));
}

/* ************************************************************************* */
/*    Set Push Button                                                        */
/* ************************************************************************* */

int Midi2EdProc :: SetPushButton(HWND hWnd, UINT uStatus)
{
	EnableWindow(GetDlgItem(hWnd, IDM_OPEN), _uStatus & STATUS_OPENFILE);
	EnableWindow(GetDlgItem(hWnd, IDM_CONVERT), _uStatus & STATUS_CONVERT);
	EnableWindow(GetDlgItem(hWnd, IDM_ADD), _uStatus & STATUS_CONVERT);
	EnableWindow(GetDlgItem(hWnd, IDM_REMOVE), _uStatus & STATUS_CONVERT);
	EnableWindow(GetDlgItem(hWnd, IDM_SAVE), _uStatus & STATUS_SAVESOURCE);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Set MIDI Channel List                                                  */
/* ************************************************************************* */

int Midi2EdProc :: SetMidiChList(LPListView pList, LPMmlData pMmlData)
{
	int				i, iMmlCh, iSmfCh, iSmfTrack;
	UINT			uStep;
	LPMmlChannel	pChannel;
	LV_ITEM			item;
	TCHAR			szBuf[STRING_BUFFER];

	memset(&item, 0 , sizeof(LV_ITEM));
	for(i = 0; i < pMmlData->GetChannelCount(); i++){
		pChannel = pMmlData->GetMmlChannel(i);
		iMmlCh = pChannel->GetMmlChannel();
		iSmfCh = pChannel->GetSmfChannel();
		iSmfTrack = pChannel->GetSmfTrack();
		uStep = (UINT)pChannel->GetStepTime();
		item.mask = LVIF_TEXT;
		item.iItem = i;
		item.pszText = szBuf;
		sprintf(szBuf, "%d", iMmlCh);
		item.iSubItem = 0;
		pList->InsertItem(&item);
		sprintf(szBuf, "%d", iSmfTrack);
		item.iSubItem = 1;
		pList->SetItem(&item);
		sprintf(szBuf, "%d", iSmfCh);
		item.iSubItem = 2;
		pList->SetItem(&item);
		IntToStrComma(uStep, szBuf);
		item.iSubItem = 3;
		pList->SetItem(&item);
	}
	return(i);
}

/* ************************************************************************* */
/*    Set Play Channel List                                                  */
/* ************************************************************************* */

int Midi2EdProc :: SetPlayChList(LPListView pList)
{
	int				i;
	LV_ITEM			item;
	TCHAR			szBuf[STRING_BUFFER];

	memset(&item, 0 , sizeof(LV_ITEM));
	for(i = 0; i < PLAYINFO_PART_MAX; i++){
		item.mask = LVIF_TEXT | LVIF_PARAM;
		item.iItem = i;
		item.pszText = szBuf;
		sprintf(szBuf, "%d", i);
		item.iSubItem = 0;
		item.lParam = PLAYLIST_CHANNEL_NONE;
		pList->InsertItem(&item);
	}
	return(i);
}

/* ************************************************************************* */
/*    Set File Name                                                          */
/* ************************************************************************* */

int Midi2EdProc :: SetFileName(LPCTSTR pszFileName)
{
	int	iLength;

	FreeFileName();
	iLength = strlen(pszFileName);
	_pszFileName = new TCHAR[iLength + 1];
	strcpy(_pszFileName, pszFileName);
	return(iLength);
}

/* ************************************************************************* */
/*    Get File Name                                                          */
/* ************************************************************************* */

inline LPCTSTR Midi2EdProc :: GetFileName(void)
{
	return(_pszFileName);
}

/* ************************************************************************* */
/*    Free File Name                                                         */
/* ************************************************************************* */

LPCTSTR Midi2EdProc :: FreeFileName(void)
{
	_DELETEARRAY(_pszFileName);
	return(_pszFileName);
}

/* ************************************************************************* */
/*    Set Window Name                                                        */
/* ************************************************************************* */

int Midi2EdProc :: SetWindowFileName(HWND hWnd, LPCTSTR pszFileName)
{
	TCHAR		szWindowName[STRING_BUFFER];

	sprintf(szWindowName, "%s - %s", GetWindowName(), pszFileName);
	SetWindowText(hWnd, szWindowName);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Check Format                                                           */
/* ************************************************************************* */

BOOL Midi2EdProc :: CheckFormat(int iFormat)
{
	BOOL bResult = FALSE;

	switch(iFormat){
		case 0:
		case 1:
			bResult = TRUE;
	}
	return(bResult);
}

/* ************************************************************************* */
/*    Check Time Base                                                        */
/* ************************************************************************* */

BOOL Midi2EdProc :: CheckTimeBase(int iTimeBase)
{
	BOOL bResult = FALSE;

	switch(iTimeBase){
		case 48:
		case 96:
		case 240:
		case 480:
			bResult = TRUE;
	}
	return(bResult);
}

/* ************************************************************************* */
/*    Get Open File Name                                                     */
/* ************************************************************************* */

int Midi2EdProc :: GetOpenFileName(HWND hWnd, LPTSTR szFileName, int iLength)
{
	int				iRet = WINPROC_CANCEL;
	TCHAR			szTemp[STRING_BUFFER];
	TCHAR			szOfnFilter[STRING_BUFFER];
	TCHAR			szOfnDefExt[STRING_BUFFER];
	OPENFILENAME	ofn;

	memset(szOfnFilter, 0, STRING_BUFFER);
	memset(szOfnDefExt, 0, STRING_BUFFER);
	memset(szTemp, 0, STRING_BUFFER);
	LoadString(GetInstance(), IDS_OFNFILTER_MID, szOfnFilter, STRING_BUFFER);
	LoadString(GetInstance(), IDS_OFNDEFEXT_MID, szOfnDefExt, STRING_BUFFER);
	if(szFileName)
		lstrcpy(szTemp, szFileName);

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hWnd;
	ofn.lpstrFilter = szOfnFilter;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = szTemp;
	ofn.nMaxFile = STRING_BUFFER;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.lpstrDefExt = szOfnDefExt;
	if(:: GetOpenFileName(&ofn)){
		iRet = WINPROC_COMPLETE;
		lstrcpyn(szFileName, szTemp, iLength);
	}
	return(iRet);
}

/* ************************************************************************* */
/*    Get Save File Name                                                     */
/* ************************************************************************* */

int Midi2EdProc :: GetSaveFileName(HWND hWnd, LPTSTR szFileName, int iLength)
{
	int				iRet;
	HANDLE			hFile;
	OPENFILENAME	ofn;
	TCHAR			szTemp[STRING_BUFFER];
	TCHAR			szFileExist[STRING_BUFFER], szText[STRING_BUFFER];
	TCHAR			szOfnFilter[STRING_BUFFER], szOfnDefExt[STRING_BUFFER];

	memset(szOfnFilter, 0, STRING_BUFFER);
	memset(szOfnDefExt, 0, STRING_BUFFER);
	memset(szTemp, 0, STRING_BUFFER);
	LoadString(_hInst, IDS_OFNFILTER_TXT, szOfnFilter, STRING_BUFFER);
	LoadString(_hInst, IDS_OFNDEFEXT_TXT, szOfnDefExt, STRING_BUFFER);
	LoadString(_hInst, IDS_FILEEXIST, szFileExist, STRING_BUFFER);
	if(szFileName)
		lstrcpy(szTemp, szFileName);

	do{
		iRet = IDYES;
		memset(&ofn, 0, sizeof(OPENFILENAME));
		ofn.lStructSize = sizeof(OPENFILENAME);
		ofn.hwndOwner = hWnd;
		ofn.lpstrFilter = szOfnFilter;
		ofn.nFilterIndex = 1;
		ofn.lpstrFile = szTemp;
		ofn.nMaxFile = STRING_BUFFER;
		ofn.Flags = OFN_HIDEREADONLY;
		ofn.lpstrDefExt = szOfnDefExt;
		if(!:: GetSaveFileName(&ofn)){
			iRet = IDCANCEL;
			break;
		}
		hFile = CreateFile(ofn.lpstrFile, GENERIC_READ, 0, NULL, OPEN_EXISTING,
			FILE_ATTRIBUTE_NORMAL, NULL);
		CloseHandle(hFile);
		if(hFile == INVALID_HANDLE_VALUE) break;
		wsprintf(szText, "%s\n\n%s", ofn.lpstrFile, szFileExist);
		iRet = MessageBox(hWnd, szText, GetWindowName(),
			MB_ICONEXCLAMATION | MB_YESNOCANCEL | MB_DEFBUTTON2);
	}while(iRet == IDNO);

	if(iRet == IDYES)
		lstrcpyn(szFileName, szTemp, iLength);
	return(iRet == IDYES ? WINPROC_COMPLETE : WINPROC_CANCEL);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_OPEN                                                  */
/* ************************************************************************* */

int Midi2EdProc :: CmdOpen(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	TCHAR			szFileName[STRING_BUFFER];
	TCHAR			szBuf[STRING_BUFFER], szBuf2[STRING_BUFFER];
	TCHAR			szDrive[_MAX_DRIVE], szDir[_MAX_DIR];
	TCHAR			szFName[_MAX_FNAME], szExt[_MAX_EXT];
	HANDLE			hFile;
	LPFileIO		pFileIO;
	DWORD			dwFileSize;
	LPBYTE			pFile;

	// MIDIt@C̎擾
	memset(szFileName, 0, sizeof(szFileName));
	if(GetFileName())
		strcpy(szFileName, GetFileName());
	if(GetOpenFileName(hWnd, szFileName, sizeof(szFileName)))
		return(WINPROC_FALSE);

	SetWindowFileName(hWnd, szFileName);

	// MIDIt@C̏
	InitMidiInfo(&_midiinfo);
	SetMidiInfo(hWnd, &_midiinfo);

	// EDPLAY\[Xݒ̏
	InitPlayInfo(&_playinfo);
	SetPlayInfo(hWnd, &_playinfo);

	// MIDI`lXg̏
	InitMidiChList(&_midilist);

	// PLAY`lXg̏
	InitPlayChList(&_playlist);

	// EDPLAY\[Xr[̏
	InitSourceView(_source);
	SetSourceView(hWnd, _source);

	// Xe[^X̏
	_uStatus = STATUS_OPENFILE;
	SetPushButton(hWnd, _uStatus);

	// MIDIt@C̓ǂݍ
	pFileIO = new FileIO;
	hFile = pFileIO->OpenFile(szFileName);
	dwFileSize = pFileIO->GetFileSize(hFile);
	pFile = new BYTE[dwFileSize];
	pFileIO->ReadFile(hFile, pFile, dwFileSize);
	pFileIO->CloseHandle(hFile);
	_DELETE(pFileIO);

	// MIDIf[^̓ǂݍ
	_appli.FreeMidiData(_pSmfData);
	_pSmfData = _appli.ReadMidiData(pFile, dwFileSize);
	_DELETEARRAY(pFile);
	if(!_pSmfData){
		LoadString(GetInstance(), IDS_NOT_MIDIFILE, szBuf, STRING_BUFFER);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		return(WINPROC_FALSE);
	}

 	// MIDIf[^MMLf[^֕ϊ
	_appli.FreeMmlData(_pMmlData);
	_pMmlData = _appli.ConvertMidiToMml(_pSmfData);

	// MIDIt@C̐ݒ
	SetFileName(szFileName);
	_splitpath(szFileName, szDrive, szDir, szFName, szExt);
	sprintf(_midiinfo.szFileName, "%s.%s", szFName, szExt);
	_midiinfo.iFormat = _pSmfData->GetFormat();
	_midiinfo.iTimeBase = _pSmfData->GetDivision();
	_midiinfo.iChannelCount = _appli.GetChannelCount(_pMmlData);
	SetMidiInfo(hWnd, &_midiinfo);

	// FORMAT̃`FbN
	if(!CheckFormat(_midiinfo.iFormat)){
		LoadString(GetInstance(), IDS_ILLEGAL_FORMAT, szBuf2, STRING_BUFFER);
		sprintf(szBuf, "FILE NAME = %s\nFORMAT = %d\n\n%s",
			_midiinfo.szFileName, _midiinfo.iFormat, szBuf2);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		return(WINPROC_FALSE);
	}

	// TIME BASẼ`FbN
	if(!CheckTimeBase(_midiinfo.iTimeBase)){
		LoadString(GetInstance(), IDS_TIMEBASE_ERROR, szBuf2, STRING_BUFFER);
		sprintf(szBuf, "FILE NAME = %s\nTIME BASE = %d\n\n%s",
			_midiinfo.szFileName, _midiinfo.iTimeBase, szBuf2);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		return(WINPROC_FALSE);
	}

	// Xebv̐ݒ
	_playinfo.iDivision = _midiinfo.iTimeBase * 4;
	SetPlayInfo(hWnd, &_playinfo);

	// MIDI`lXg̐ݒ
	if(SetMidiChList(&_midilist, _pMmlData))
		_midilist.SetFocusedItem(0);

	// EDPLAY`lXg̐ݒ
	if(SetPlayChList(&_playlist))
		_playlist.SetFocusedItem(0);

	// \[Xt@Cl[̐ݒ
	LoadString(_hInst, IDS_OFNDEFEXT_TXT, szExt, STRING_BUFFER);
	sprintf(_playinfo.szFileName, "%s.%s", szFName, szExt);

	// Xe[^X̏
	_uStatus |= STATUS_CONVERT;
	SetPushButton(hWnd, _uStatus);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_ADD                                                   */
/* ************************************************************************* */

int Midi2EdProc :: CmdAdd(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int				iMmlCh, iSmfCh, iSmfTrack, iMidiIndex, iPlayIndex;
	UINT			uStep;
	LPMmlChannel	pChannel;
	LV_ITEM			item;
	TCHAR			szBuf[STRING_BUFFER];

	memset(&item, 0 , sizeof(LV_ITEM));
	iMidiIndex = _midilist.GetFocusedItem();
	iPlayIndex = _playlist.GetFocusedItem();
	pChannel = _pMmlData->GetMmlChannel(iMidiIndex);
	iMmlCh = pChannel->GetMmlChannel();
	iSmfCh = pChannel->GetSmfChannel();
	iSmfTrack = pChannel->GetSmfTrack();
	uStep = (UINT)pChannel->GetStepTime();
	item.iItem = iPlayIndex;
	item.mask = LVIF_PARAM;
	item.iSubItem = 0;
	item.lParam = iMmlCh;
	_playlist.SetItem(&item);
	item.mask = LVIF_TEXT;
	item.pszText = szBuf;
	sprintf(szBuf, "%d", iMmlCh);
	item.iSubItem = 1;
	_playlist.SetItem(&item);
	sprintf(szBuf, "%d", iSmfTrack);
	item.iSubItem = 2;
	_playlist.SetItem(&item);
	sprintf(szBuf, "%d", iSmfCh);
	item.iSubItem = 3;
	_playlist.SetItem(&item);
	IntToStrComma(uStep, szBuf);
	item.iSubItem = 4;
	_playlist.SetItem(&item);

	if(iMidiIndex + 1 < _midilist.GetItemCount())
		_midilist.SetFocusedItem(iMidiIndex + 1);
	if(iPlayIndex + 1 < _playlist.GetItemCount())
		_playlist.SetFocusedItem(iPlayIndex + 1);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_REMOVE                                                */
/* ************************************************************************* */

int Midi2EdProc :: CmdRemove(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int			iPlayIndex;
	LV_ITEM		item;

	memset(&item, 0 , sizeof(LV_ITEM));
	iPlayIndex = _playlist.GetFocusedItem();
	item.iItem = iPlayIndex;
	item.mask = LVIF_PARAM;
	item.iSubItem = 0;
	item.lParam = PLAYLIST_CHANNEL_NONE;
	_playlist.SetItem(&item);
	item.mask = LVIF_TEXT;
	item.pszText = "";
	item.iSubItem = 1;
	_playlist.SetItem(&item);
	item.iSubItem = 2;
	_playlist.SetItem(&item);
	item.iSubItem = 3;
	_playlist.SetItem(&item);
	item.iSubItem = 4;
	_playlist.SetItem(&item);

	if(iPlayIndex + 1 < _playlist.GetItemCount())
		_playlist.SetFocusedItem(iPlayIndex + 1);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_CONVERT                                               */
/* ************************************************************************* */

int Midi2EdProc :: CmdConvert(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int			i, iLength, iResolution, iDivision, iMultiple;
	int			iPart, iLine, iPlay, iChannel;
	TCHAR		szBuf[STRING_BUFFER], szBuf2[STRING_BUFFER];
	TCHAR		szPlay[STRING_BUFFER];
	string		init, source;
	LPEdTrans	pEdTrans;

	static int	iLenDef[] = {MMLLEN_1, MMLLEN_2, MMLLEN_2, MMLLEN_4, MMLLEN_4,
						MMLLEN_8, MMLLEN_6, MMLLEN_8, MMLLEN_16, MMLLEN_12,
						MMLLEN_16, MMLLEN_32, MMLLEN_24, MMLLEN_32, MMLLEN_48,
						MMLLEN_64, MMLLEN_96, MMLLEN_192};

	// ŏ\̎擾
	iMultiple = _midiinfo.iTimeBase / MMLMINDIVRATIO;
	iLength = SendMessage(GetDlgItem(hWnd, IDC_RESOLUTION), CB_GETCURSEL, 0,0);
	iResolution = iMultiple * LENGTHTIME(iLength);

	// Xebv̎擾
	GetWindowText(GetDlgItem(hWnd, IDC_DIVISION), szBuf, sizeof(szBuf));
	iDivision = StrToIntComma(szBuf);
	if(iDivision < PLAYINFO_DIVISION_MIN || iDivision > PLAYINFO_DIVISION_MAX){
		LoadString(GetInstance(), IDS_ILLEGAL_DIVISION, szBuf2, STRING_BUFFER);
		sprintf(szBuf, "Xebv = %d\n\n%s", iDivision, szBuf2);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		return(WINPROC_FALSE);
	}

	// gpp[g̎擾
	GetWindowText(GetDlgItem(hWnd, IDC_EDPLAYCHANNEL), szBuf, sizeof(szBuf));
	iPart = StrToIntComma(szBuf);
	if(iPart < PLAYINFO_PART_MIN || iPart > PLAYINFO_PART_MAX){
		LoadString(GetInstance(), IDS_ILLEGAL_PART, szBuf2, STRING_BUFFER);
		sprintf(szBuf, "gpp[g = %d\n\n%s", iPart, szBuf2);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		return(WINPROC_FALSE);
	}

	// Jnsԍ
	GetWindowText(GetDlgItem(hWnd, IDC_LINENUMBER), szBuf, sizeof(szBuf));
	iLine = StrToIntComma(szBuf);
	if(iLine < PLAYINFO_LINENUMBER_MIN || iLine > PLAYINFO_LINENUMBER_MAX){
		LoadString(GetInstance(), IDS_ILLEGAL_LINE, szBuf2, STRING_BUFFER);
		sprintf(szBuf, "Jnsԍ = %d\n\n%s", iLine, szBuf2);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		return(WINPROC_FALSE);
	}

	// PLAY̎擾
	iPlay = SendMessage(GetDlgItem(hWnd, IDC_PLAYTYPE), CB_GETCURSEL, 0,0);
	SendMessage(GetDlgItem(hWnd, IDC_PLAYTYPE), CB_GETLBTEXT,
		(WPARAM)iPlay, (LPARAM)szPlay);

 	// MIDIf[^MMLf[^֕ϊ
	_appli.FreeMmlData(_pMmlData);
	_pMmlData = _appli.ConvertMidiToMml(_pSmfData);

	// ŏ\֑
	_appli.Normalize(_pMmlData, iResolution);

	// ߂̋؂}
	_appli.Divide(_pMmlData, iDivision);

	// Xbev炎֕ϊ
	_appli.ConvertLength(_pMmlData);

	// EDPLAY\[Xr[̏
	InitSourceView(_source);
	SetSourceView(hWnd, _source);

	// EDPLAY̍쐬
	init = _appli.CreateSourceInit(iLine, PLAYINFO_LINENUMBER_STEP);

	// EDPLAY\[X֕ϊ
	pEdTrans = new EdTrans[iPart];
	for(i = 0; i < iPart; i++){
		iChannel = _playlist.GetParam(i);
		if(iChannel == PLAYLIST_CHANNEL_NONE)
			pEdTrans[i].SetChannel(MMLCHANNEL_NOUSE);
		else{
			iLength = _appli.GetMaxLengthCount(_pMmlData, iChannel);
			pEdTrans[i].SetChannel(iChannel);
			pEdTrans[i].SetOctave(PLAY_OCTAVE);
			pEdTrans[i].SetLength(iLenDef[iLength]);
		}
	}
	source = _appli.CreateSource(_pMmlData, pEdTrans, iPart,
		iLine + _appli.GetInitLine() * PLAYINFO_LINENUMBER_STEP,
		PLAYINFO_LINENUMBER_STEP, szPlay);
	_source = init + source;
	_DELETEARRAY(pEdTrans);

	// EDPLAY\[X̕\
	SetSourceView(hWnd, _source);

	// Xe[^X̏
	_uStatus |= STATUS_SAVESOURCE;;
	SetPushButton(hWnd, _uStatus);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_SAVE                                                  */
/* ************************************************************************* */

int Midi2EdProc :: CmdSave(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int			iLength;
	LPTSTR		pszText;
	TCHAR		szFileName[STRING_BUFFER], szBuf[STRING_BUFFER];
	HANDLE		hFile;
	LPFileIO	pFileIO;

	// Source ViewҏW̃eLXg̎擾
	iLength = GetWindowTextLength(GetDlgItem(hWnd, IDC_SOURCE));
	pszText = new TCHAR[iLength + 1];
	iLength = GetWindowText(GetDlgItem(hWnd, IDC_SOURCE), pszText, iLength +1);
	if(!iLength){
		LoadString(GetInstance(), IDS_EMPTY_TEXT, szBuf, STRING_BUFFER);
		MessageBox(hWnd, szBuf, GetWindowName(), MB_OK | MB_ICONEXCLAMATION);
		_DELETEARRAY(pszText);
		return(WINPROC_FALSE);
	}

	// ۑt@C̎擾
	memset(szFileName, 0, sizeof(szFileName));
	if(_playinfo.szFileName)
		strcpy(szFileName, _playinfo.szFileName);
	if(GetSaveFileName(hWnd, szFileName, sizeof(szFileName)))
		return(WINPROC_FALSE);

	// t@C̕ۑ
	pFileIO = new FileIO;
	hFile = pFileIO->OpenFile(szFileName,
		GENERIC_WRITE, FILE_ATTRIBUTE_NORMAL, CREATE_ALWAYS);
	pFileIO->WriteFile(hFile, (LPVOID)pszText, iLength);
	pFileIO->CloseHandle(hFile);
	_DELETE(pFileIO);
	_DELETEARRAY(pszText);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND                                                             */
/* ************************************************************************* */

int Midi2EdProc :: Command(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam)){
		case IDM_OPEN:	// -------------------------------------- IDM_OPEN ----
			CmdOpen(hWnd, wParam, lParam);
			break;

		case IDM_ADD:	// --------------------------------------- IDM_ADD ----
			CmdAdd(hWnd, wParam, lParam);
			break;

		case IDM_REMOVE:// ------------------------------------ IDM_REMOVE ----
			CmdRemove(hWnd, wParam, lParam);
			break;

		case IDM_CONVERT:// ---------------------------------- IDM_CONVERT ----
			CmdConvert(hWnd, wParam, lParam);
			break;

		case IDM_SAVE:// ---------------------------------- IDM_CONVERT ----
			CmdSave(hWnd, wParam, lParam);
			break;

//		case IDM_DELREGISTORY:	// ---------------------- IDM_DELREGISTORY ----
//			CmdDelRegistory(hWnd, wParam, lParam);
//			break;

		case IDM_ABOUT:	// ------------------------------------- IDM_ABOUT ----
			CmdAbout(hWnd, wParam, lParam);
			break;

#ifdef	APPLI_USE_HELP
		case IDM_HELP:	// -------------------------------------- IDM_HELP ----
			WinHelp(hWnd, _pszHelpPath, HELP_FINDER, 0);
			break;
#endif

		case IDM_EXIT:	// -------------------------------------- IDM_EXIT ----
			DestroyWindow(hWnd);
			break;
	}
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_ACTIVATE                                                            */
/* ************************************************************************* */

int Midi2EdProc :: Activate(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam)){
		case WA_ACTIVE:
		case WA_CLICKACTIVE:
			_pKeyAccel->SetWindowHandle(hWnd);
			_pKeyAccel->SetAcceleratorHandle(_hAccel);
			SetFocus(_hFocus);
			break;

		case WA_INACTIVE:
			_hFocus = GetFocus();
			break;
	}
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_INITMENU                                                            */
/* ************************************************************************* */

int Midi2EdProc :: InitMenu(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	HMENU	hMenu;
	UINT	uEnable, uDisable;

	hMenu = (HMENU)wParam;
	CheckMenuItem(hMenu, IDM_DELREGISTORY,
		MF_BYCOMMAND | _bDelReg ? MF_CHECKED : MF_UNCHECKED);

	uEnable = MF_BYCOMMAND | MF_ENABLED;
	uDisable = MF_BYCOMMAND | MF_DISABLED | MF_GRAYED;
	if(_uStatus & STATUS_OPENFILE)
		EnableMenuItem(hMenu, IDM_OPEN, uEnable);
	else
		EnableMenuItem(hMenu, IDM_OPEN, uDisable);

	if(_uStatus & STATUS_CONVERT){
		EnableMenuItem(hMenu, IDM_CONVERT, uEnable);
		EnableMenuItem(hMenu, IDM_ADD, uEnable);
		EnableMenuItem(hMenu, IDM_REMOVE, uEnable);
	}
	else{
		EnableMenuItem(hMenu, IDM_CONVERT, uDisable);
		EnableMenuItem(hMenu, IDM_ADD, uDisable);
		EnableMenuItem(hMenu, IDM_REMOVE, uDisable);
	}

	if(_uStatus & STATUS_SAVESOURCE)
		EnableMenuItem(hMenu, IDM_SAVE, uEnable);
	else
		EnableMenuItem(hMenu, IDM_SAVE, uDisable);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_CREATE Appendix                                                     */
/* ************************************************************************* */

int Midi2EdProc :: CreateAppendix(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	HWND	hSpin;
	UDACCEL accel;

	// ======== Centering Window ========
	CenteringWindow(hWnd);

	// ======== Initialize Control ========
	// \R{BOX̏
	InitCombobox(GetDlgItem(hWnd, IDC_RESOLUTION),
		RESOLUTION_COUNT, IDS_RESOLUTION00);

	// PLAYR{BOX̏
	InitCombobox(GetDlgItem(hWnd, IDC_PLAYTYPE),
		PLAYTYPE_COUNT, IDS_PLAYTYPE00);

	// gp`lXsRg[쐬
	hSpin = CreateUpDownControl(WS_CHILD | WS_BORDER | WS_VISIBLE
		| UDS_SETBUDDYINT | UDS_ALIGNRIGHT,	285, 15, 15, 12,
		hWnd, IDCS_EDPLAYCHANNEL, GetInstance(),
		GetDlgItem(hWnd, IDC_EDPLAYCHANNEL), PLAYINFO_PART_MAX,
		PLAYINFO_PART_MIN, PLAYINFO_PART);

	// XebvXsRg[쐬
	hSpin = CreateUpDownControl(WS_CHILD | WS_BORDER | WS_VISIBLE
		| UDS_SETBUDDYINT | UDS_ALIGNRIGHT,	285, 45, 15, 12,
		hWnd, IDCS_DIVISION, GetInstance(), GetDlgItem(hWnd, IDC_DIVISION),
			PLAYINFO_DIVISION_MAX, PLAYINFO_DIVISION_MIN, PLAYINFO_DIVISION);
	accel.nSec = 1, accel.nInc = MMLMINDIVRATIO;
	SendMessage(hSpin, UDM_SETACCEL, 1, (LPARAM)&accel);

	// JnsԍXsRg[쐬
	hSpin = CreateUpDownControl(WS_CHILD | WS_BORDER | WS_VISIBLE
		| UDS_SETBUDDYINT | UDS_ALIGNRIGHT,	285, 75, 15, 12,
		hWnd, IDCS_LINENUMBER, GetInstance(), GetDlgItem(hWnd, IDC_LINENUMBER),
		PLAYINFO_LINENUMBER_MAX, PLAYINFO_LINENUMBER_MIN, PLAYINFO_LINENUMBER);
	accel.nSec = 1, accel.nInc = PLAYINFO_LINENUMBER_STEP;
	SendMessage(hSpin, UDM_SETACCEL, 1, (LPARAM)&accel);

	// MIDI`lXg̏
	_midilist.Initialize(hWnd, _hInst);
	_midilist.SetWindowHandle(GetDlgItem(hWnd, IDC_MIDICHLIST));
	_midilist.LoadColumn(MAKEINTRESOURCE(IDR_MIDILIST_COLUMN));

	// PLAY`lXg̏
	_playlist.Initialize(hWnd, _hInst);
	_playlist.SetWindowHandle(GetDlgItem(hWnd, IDC_PLAYCHLIST));
	_playlist.LoadColumn(MAKEINTRESOURCE(IDR_PLAYLIST_COLUMN));

	// ======== Initialize Parameter ========
	// MIDIt@C̏
	InitMidiInfo(&_midiinfo);
	SetMidiInfo(hWnd, &_midiinfo);

	// EDPLAY\[Xݒ̏
	InitPlayInfo(&_playinfo);
	SetPlayInfo(hWnd, &_playinfo);

	// MIDI`lXg̏
	InitMidiChList(&_midilist);

	// PLAY`lXg̏
	InitPlayChList(&_playlist);

	// EDPLAY\[Xr[̏
	InitSourceView(_source);
	SetSourceView(hWnd, _source);

	// Xe[^X̏
	_uStatus = STATUS_OPENFILE;
	SetPushButton(hWnd, _uStatus);

	// tH[JX̐ݒ
	SetFocus(GetDlgItem(hWnd, IDM_OPEN));

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_DESTROY Appendix                                                    */
/* ************************************************************************* */

int Midi2EdProc :: DestroyAppendix(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	return(WINPROC_COMPLETE);
}

