/* ********************************************* */
/*    MIDI to EDPLAY Procedure Class ver.1.00    */
/*                                 'm2eproc.h'   */
/*                              by T.Nakayama    */
/* ********************************************* */

#ifndef	_CLASS_MIDI2EDPLAY_PROCEDURE_
#define _CLASS_MIDI2EDPLAY_PROCEDURE_

#include <string>
using namespace std;

#include "wdlgproc.h"
#include "listview.h"
#include "glblib.h"
#include "fileio.h"
#include "mmlevent.h"

/* **************************************************** Constance Value **** */

#define MIDIINFO_FORMAT				0
#define MIDIINFO_TIMEBASE			480
#define MIDIINFO_CHANNEL			0

#define PLAYINFO_PART				6
#define PLAYINFO_RESOLUTION			13
#define PLAYINFO_DIVISION			1920
#define PLAYINFO_LINENUMBER			1000
#define PLAYINFO_PLAYTYPE			0

#define PLAYINFO_PART_MIN			1
#define PLAYINFO_PART_MAX			18
#define PLAYINFO_DIVISION_MIN		1
#define PLAYINFO_DIVISION_MAX		1920
#define PLAYINFO_LINENUMBER_MIN		1
#define PLAYINFO_LINENUMBER_MAX		32000
#define PLAYINFO_LINENUMBER_STEP	10

#define STATUS_OPENFILE				0x01
#define STATUS_CONVERT				0x02
#define STATUS_SAVESOURCE			0x04

#define PLAY_OCTAVE					4

#define PLAYLIST_CHANNEL_NONE		0xffffffffl

/* ********************************************************** Structure **** */

typedef struct MIDIINFO_tag{
	TCHAR	szFileName[STRING_BUFFER];
	int		iFormat;
	int		iTimeBase;
	int		iChannelCount;
} MIDIINFO;
typedef MIDIINFO*	LPMIDIINFO;

typedef struct PLAYINFO_tag{
	int		iPartCount;
	int		iResolution;
	int		iDivision;
	int		iLineNumber;
	int		iPlayType;
	TCHAR	szFileName[STRING_BUFFER];
} PLAYINFO;
typedef PLAYINFO*	LPPLAYINFO;

/* ************************************************************** Class **** */

class Midi2EdProc : public WinDlgProc{
private:
	Appli			_appli;
	LPSmfData		_pSmfData;
	LPMmlData		_pMmlData;
	ListView		_midilist, _playlist;
	MIDIINFO		_midiinfo;
	PLAYINFO		_playinfo;
	string			_source;
	UINT			_uStatus;
	LPTSTR			_pszFileName;
	HWND			_hFocus;

public:
						Midi2EdProc();				// Constructor
	virtual				~Midi2EdProc();				// Destructor

	virtual int			CenteringWindow(HWND);

	virtual int			InitCombobox(HWND, int, UINT);

	virtual int			InitMidiInfo(LPMIDIINFO);
	virtual int			InitPlayInfo(LPPLAYINFO);
	virtual int			InitMidiChList(LPListView);
	virtual int			InitPlayChList(LPListView);
	virtual string		InitSourceView(string&);

	virtual int			SetMidiInfo(HWND, LPMIDIINFO);
	virtual int			SetPlayInfo(HWND, LPPLAYINFO);
	virtual int			SetSourceView(HWND, string&);
	virtual int			SetPushButton(HWND, UINT);
	virtual int			SetMidiChList(LPListView, LPMmlData);
	virtual int			SetPlayChList(LPListView);

	virtual	int			SetFileName(LPCTSTR);		// Set File Name
	virtual	LPCTSTR		GetFileName(void);			// Get File Name
	virtual	LPCTSTR		FreeFileName(void);			// Free File Name
	virtual	int			SetWindowFileName(HWND, LPCTSTR);

	virtual BOOL		CheckFormat(int);
	virtual BOOL		CheckTimeBase(int);

	virtual int			GetOpenFileName(HWND, LPTSTR, int);
	virtual int			GetSaveFileName(HWND, LPTSTR, int);

	virtual int			CmdOpen(HWND, WPARAM, LPARAM);
	virtual int			CmdAdd(HWND, WPARAM, LPARAM);
	virtual int			CmdRemove(HWND, WPARAM, LPARAM);
	virtual int			CmdConvert(HWND, WPARAM, LPARAM);
	virtual int			CmdSave(HWND, WPARAM, LPARAM);
	virtual int			Command(HWND, WPARAM, LPARAM);
	virtual int			Activate(HWND, WPARAM, LPARAM);
	virtual int			InitMenu(HWND, WPARAM, LPARAM);
//	virtual int			Size(HWND, WPARAM, LPARAM);
//	virtual int			Create(HWND, WPARAM, LPARAM);
	virtual int			CreateAppendix(HWND, WPARAM, LPARAM);
//	virtual int			Destroy(HWND, WPARAM, LPARAM);
	virtual int			DestroyAppendix(HWND, WPARAM, LPARAM);
//	virtual int			Close(HWND, WPARAM, LPARAM);
//	virtual LRESULT		Procedure(HWND, UINT, WPARAM, LPARAM);
};
typedef Midi2EdProc*		LPMidi2EdProc;

#endif	// _CLASS_MIDI2EDPLAY_PROCEDURE_
