/* ************************************************* */
/*    Music Macro Language Channel Class ver.1.00    */
/*                                     'mmlch.cpp'   */
/*                                  by T.Nakayama    */
/* ************************************************* */

#include "mmlch.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

MmlChannel :: MmlChannel()
 :	_iSmfTrack(0), _iSmfChannel(0), _iMmlChannel(0), _iBarCount(0)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

MmlChannel :: ~MmlChannel()
{
	DeleteAllEvents();
}

/* ************************************************************************* */
/*    Set MML Channel Information                                            */
/* ************************************************************************* */

int MmlChannel :: SetInfo(LPMMLCHANNELINFO pInfo)
{
	SetSmfTrack(pInfo->iSmfTrack);
	SetSmfChannel(pInfo->iSmfChannel);
	SetMmlChannel(pInfo->iMmlChannel);
	return(MMLCHANNEL_COMPLETE);
}

/* ************************************************************************* */
/*    Get MML Channel Information                                            */
/* ************************************************************************* */

int MmlChannel :: GetInfo(LPMMLCHANNELINFO pInfo)
{
	pInfo->iSmfTrack = GetSmfTrack();
	pInfo->iSmfChannel = GetSmfChannel();
	pInfo->iMmlChannel = GetMmlChannel();
	return(MMLCHANNEL_COMPLETE);
}

/* ************************************************************************* */
/*    Set SMF Track                                                          */
/* ************************************************************************* */

int MmlChannel :: SetSmfTrack(int iSmfTrack)
{
	int		iSmfTrackOld;

	iSmfTrackOld = _iSmfTrack;
	_iSmfTrack = iSmfTrack;
	return(iSmfTrackOld);
}

/* ************************************************************************* */
/*    Get SMF Track                                                          */
/* ************************************************************************* */

inline int MmlChannel :: GetSmfTrack(void)
{
	return(_iSmfTrack);
}

/* ************************************************************************* */
/*    Set SMF Channel                                                        */
/* ************************************************************************* */

int MmlChannel :: SetSmfChannel(int iSmfChannel)
{
	int		iSmfChannelOld;

	iSmfChannelOld = _iSmfChannel;
	_iSmfChannel = iSmfChannel;
	return(iSmfChannelOld);
}

/* ************************************************************************* */
/*    Get SMF Channel                                                        */
/* ************************************************************************* */

inline int MmlChannel :: GetSmfChannel(void)
{
	return(_iSmfChannel);
}

/* ************************************************************************* */
/*    Set MML Channel                                                        */
/* ************************************************************************* */

int MmlChannel :: SetMmlChannel(int iMmlChannel)
{
	int		iMmlChannelOld;

	iMmlChannelOld = _iMmlChannel;
	_iMmlChannel = iMmlChannel;
	return(iMmlChannelOld);
}

/* ************************************************************************* */
/*    Get MML Channel                                                        */
/* ************************************************************************* */

inline int MmlChannel :: GetMmlChannel(void)
{
	return(_iMmlChannel);
}

/* ************************************************************************* */
/*    Get Event Count                                                        */
/* ************************************************************************* */

inline int MmlChannel :: GetEventCount(void)
{
	return(_pMmlEvent.size());
}

/* ************************************************************************* */
/*    Get Bar Count                                                          */
/* ************************************************************************* */

inline int MmlChannel :: GetBarCount(void)
{
	return(_iBarCount);
}

/* ************************************************************************* */
/*    Get Step Time                                                          */
/* ************************************************************************* */

DWORD MmlChannel :: GetStepTime(void)
{
	DWORD				dwStepTime = 0;
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	for(ite = GetBegin(); ite != GetEnd(); ite++){
		pEvent = *ite;
		dwStepTime += pEvent->GetDeltaTime();
	}
	return(dwStepTime);
}

/* ************************************************************************* */
/*    Get Length Count                                                       */
/* ************************************************************************* */

int MmlChannel :: GetLengthCount(int iLength)
{
	int					iCount = 0;
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	for(ite = GetBegin(); ite != GetEnd(); ite++){
		pEvent = *ite;
		if(pEvent->GetLength() == iLength)
			iCount++;
	}
	return(iCount);
}

/* ************************************************************************* */
/*    Get Max Length Count                                                   */
/* ************************************************************************* */

int MmlChannel :: GetMaxLengthCount(void)
{
	int					i, iMaxLength = 0, iLength[MMLLEN_COUNT];
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;
	static int			iTable[MMLLEN_COUNT]\
					= {	MMLLEN_1, MMLLEN_2, MMLLEN_2, MMLLEN_4, MMLLEN_4,
						MMLLEN_8, MMLLEN_6, MMLLEN_8, MMLLEN_16, MMLLEN_12,
						MMLLEN_16, MMLLEN_32, MMLLEN_24, MMLLEN_32, MMLLEN_48,
						MMLLEN_64, MMLLEN_96, MMLLEN_192};

	for(i = 0; i < MMLLEN_COUNT; i++)
		iLength[i] = 0;
	for(ite = GetBegin(); ite != GetEnd(); ite++){
		pEvent = *ite;
		if(pEvent->GetLength() != MMLLEN_NONE)
			iLength[iTable[pEvent->GetLength()]]++;
	}
	for(i = 0; i < MMLLEN_COUNT; i++)
		if(iLength[iMaxLength] < iLength[i])
			iMaxLength = i;

	return(iMaxLength);
}

/*
int MmlChannel :: GetMaxLengthCount(void)
{
	int					i, iMaxLength = 0, iLength[MMLLEN_COUNT];
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	for(i = 0; i < MMLLEN_COUNT; i++)
		iLength[i] = 0;
	for(ite = GetBegin(); ite != GetEnd(); ite++){
		pEvent = *ite;
		if(pEvent->GetLength() != MMLLEN_NONE)
			iLength[pEvent->GetLength()]++;
	}
	for(i = 0; i < MMLLEN_COUNT; i++)
		if(iLength[iMaxLength] < iLength[i])
			iMaxLength = i;

	return(iMaxLength);
}
*/

/* ************************************************************************* */
/*    Get Begin Event                                                        */
/* ************************************************************************* */

inline list < LPMmlEvent > :: iterator MmlChannel :: GetBegin(void)
{
	return(_pMmlEvent.begin());
}

/* ************************************************************************* */
/*    Get End of Event                                                       */
/* ************************************************************************* */

inline list < LPMmlEvent > :: iterator MmlChannel :: GetEnd(void)
{
	return(_pMmlEvent.end());
}

/* ************************************************************************* */
/*    Insert Event                                                           */
/* ************************************************************************* */

inline list < LPMmlEvent > :: iterator MmlChannel :: Insert(
	list < LPMmlEvent > :: iterator ite, LPMmlEvent pEvent)
{
	return(_pMmlEvent.insert(ite, pEvent));
}

/* ************************************************************************* */
/*    Add Event                                                              */
/* ************************************************************************* */

inline list < LPMmlEvent > :: iterator MmlChannel :: Add(LPMmlEvent pEvent)
{
	return(Insert(_pMmlEvent.end(), pEvent));
}

/* ************************************************************************* */
/*    Delete Event                                                           */
/* ************************************************************************* */

list < LPMmlEvent > :: iterator MmlChannel :: Delete(
	list < LPMmlEvent > :: iterator ite)
{
	list < LPMmlEvent > :: iterator	tmp;

	tmp = ite;
	tmp++;
	_pMmlEvent.erase(ite);
	return(tmp);
}

/* ************************************************************************* */
/*    Delete All Events                                                      */
/* ************************************************************************* */

int MmlChannel :: DeleteAllEvents(void)
{
	list < LPMmlEvent > :: iterator	ite;

	for(ite = GetBegin(); ite != GetEnd(); ite++)
		ite = Delete(ite);
	return(GetEventCount());
}

/* ************************************************************************* */
/*    Normailze                                                              */
/* ************************************************************************* */

int MmlChannel :: Normalize(int iMinDiv)
{
	int					iDeltaTime, iBalance = 0, iSurplus;
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	ite = GetBegin();
	while(ite != GetEnd()){
		pEvent = *ite;
		iDeltaTime = (int)pEvent->GetDeltaTime();
		if(pEvent->GetCode() == MMLCODE_BAR){
			ite++;
			continue;
		}
		iDeltaTime += iBalance;
		iBalance = 0;
		if(iDeltaTime == 0){
			ite = Delete(ite);
			continue;
		}
		while(iSurplus = iDeltaTime % iMinDiv){
			if(iSurplus * 2 < iMinDiv){
				iDeltaTime -= iSurplus;
				iBalance += iSurplus;
			}
			else{
				iDeltaTime += (iMinDiv - iSurplus);
				iBalance -= (iMinDiv - iSurplus);
			}
		}
		pEvent->SetDeltaTime(iDeltaTime);
		if(iDeltaTime == 0)
			ite = Delete(ite);
		else
			ite++;
	}

#ifdef _DEBUG
	CheckEvent();
#endif // _DEBUG

	return(iBalance);
}

/* ************************************************************************* */
/*    Divide Bar                                                             */
/* ************************************************************************* */

int MmlChannel :: Divide(int iDivision)
{
	int					iDeltaTime, iBarTime = 0, iSurplus, iBarCount = 0;
	UINT				uCode, uVelocity;
	BOOL				bBar;
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	ite = GetBegin();
	while(ite != GetEnd()){
		bBar = FALSE;
		pEvent = *ite;
		if(pEvent->GetCode() == MMLCODE_BAR){
			bBar = TRUE;
			ite = Delete(ite);
			continue;
		}
		iDeltaTime = (int)pEvent->GetDeltaTime();
		if((iBarTime + iDeltaTime) >= iDivision){
			// 𕪊C^Ct
			iSurplus = (iBarTime + iDeltaTime) - iDivision;
			pEvent->SetDeltaTime(iDeltaTime - iSurplus);
			uCode = pEvent->GetCode();
			uVelocity = pEvent->GetVelocity();
			if(iSurplus > 0 && uCode != MMLCODE_REST)
				pEvent->SetParam(MMLPARAM_TAI);
			iBarTime = 0;
			ite++;

			// ߂̋؂}
			pEvent = new MmlEvent;
			pEvent->SetCode(MMLCODE_BAR);
			Insert(ite, pEvent);
			bBar = TRUE;
			iBarCount++;

			// ߂֎c̉}
			if(iSurplus > 0){
				// Insert Code;
				pEvent = new MmlEvent;
				pEvent->SetCode(uCode);
				pEvent->SetVelocity(uVelocity);
				pEvent->SetDeltaTime(iSurplus);
				Insert(ite, pEvent);
				iBarTime = 0;
				bBar = FALSE;
				ite--;
			}
		}
		else{
			iBarTime += iDeltaTime;
			ite++;
		}
	}
	if(!bBar){
		// ߂̋؂}
		pEvent = new MmlEvent;
		pEvent->SetCode(MMLCODE_BAR);
		Insert(ite, pEvent);
		iBarCount++;
	}
	_iBarCount = iBarCount;

#ifdef _DEBUG
	CheckEvent();
#endif // _DEBUG

	return(iBarCount);
}

/* ************************************************************************* */
/*    Convert Length                                                         */
/* ************************************************************************* */

int MmlChannel :: ConvertLength(int iTimeBase)
{
	int					iDeltaTime, iMultiple, iLength, iRemainTime, i;
	UINT				uCode, uVelocity;
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	iMultiple = iTimeBase / LENGTHTIME(MMLLEN_4);
	ite = GetBegin();
	while(ite != GetEnd()){
		pEvent = *ite;
		if(pEvent->GetCode() == MMLCODE_BAR){
			ite++;
			continue;
		}
		iDeltaTime = (int)pEvent->GetDeltaTime();
		for(i = 0;;){
			iLength = LENGTHTIME(i) * iMultiple;
			iRemainTime = iDeltaTime - iLength;
			if(iRemainTime == 0){
				pEvent->SetDeltaTime(iLength);
				pEvent->SetLength(i);
				break;
			}
			if(iRemainTime > 0){
				iDeltaTime = iRemainTime;
				pEvent->SetDeltaTime(iLength);
				pEvent->SetLength(i);
				uCode = pEvent->GetCode();
				uVelocity = pEvent->GetVelocity();
				if(uCode != MMLCODE_REST)
					pEvent->SetParam(MMLPARAM_TAI);
				ite++;
				pEvent = new MmlEvent;
				pEvent->SetCode(uCode);
				pEvent->SetVelocity(uVelocity);
				pEvent->SetDeltaTime(iDeltaTime);
				Insert(ite, pEvent);
				ite--;
			}
			if(iRemainTime < 0){
				if(i + 1 >= MMLLEN_COUNT){
					pEvent->SetDeltaTime(iLength);
					pEvent->SetLength(i);
					break;
				}
				i++;
			}
		}
		ite++;
	}

#ifdef _DEBUG
	CheckEvent();
#endif // _DEBUG

	return(MMLCHANNEL_COMPLETE);
}

#ifdef _DEBUG

/* ************************************************************************* */
/*    Check Event                                                            */
/* ************************************************************************* */

int MmlChannel :: CheckEvent(void)
{
	int					iDeltaTime, iLength;
	UINT				uCode, uVelocity, uParam;
	LPMmlEvent			pEvent;
	list < LPMmlEvent >	:: iterator	ite;

	for(ite = GetBegin(); ite != GetEnd(); ite++){
		pEvent = *ite;
		uCode = pEvent->GetCode();
		uVelocity = pEvent->GetVelocity();
		iDeltaTime = (int)pEvent->GetDeltaTime();
		uParam = (UINT)pEvent->GetParam();
		iLength = pEvent->GetLength();
	}
	return(MMLCHANNEL_COMPLETE);
}

#endif // _DEBUG
