/* ********************************************** */
/*    Music Macro Language Data Class ver.1.00    */
/*                                'mmldata.cpp'   */
/*                               by T.Nakayama    */
/* ********************************************** */

#include "mmldata.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

MmlData :: MmlData()
 :	_iChannelCount(0), _iDivision(0)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

MmlData :: ~MmlData()
{
	int		i;

	for(i=0; i<_pMmlChannel.size(); i++)
		_DELETE(_pMmlChannel[i]);
}

/* ************************************************************************* */
/*    Allocate MML Channel                                                   */
/* ************************************************************************* */

int MmlData :: AllocateChannel(int iChannelCount)
{
	int		i;

	_pMmlChannel.resize(iChannelCount);
	for(i=0; i<_pMmlChannel.size(); i++)
		_pMmlChannel[i] = new MmlChannel;
	return(_pMmlChannel.size());
}

/* ************************************************************************* */
/*    Get MML Channel                                                        */
/* ************************************************************************* */

LPMmlChannel MmlData :: GetMmlChannel(int iChannel)
{
	if((iChannel < 0) || (iChannel >= _pMmlChannel.size()))
		return(NULL);
	else
		return(_pMmlChannel[iChannel]);
}

/* ************************************************************************* */
/*    Set Channel Count                                                      */
/* ************************************************************************* */

int MmlData :: SetChannelCount(int iChannelCount)
{
	int		iChannelCountOld;

	iChannelCountOld = _iChannelCount;
	_iChannelCount = iChannelCount;
	return(iChannelCountOld);
}

/* ************************************************************************* */
/*    Get Channel Count                                                      */
/* ************************************************************************* */

inline int MmlData :: GetChannelCount(void)
{
	return(_iChannelCount);
}

/* ************************************************************************* */
/*    Set Division                                                           */
/* ************************************************************************* */

int MmlData :: SetDivision(int iDivision)
{
	int		iDivisionOld;

	iDivisionOld = _iDivision;
	_iDivision = iDivision;
	return(iDivisionOld);
}

/* ************************************************************************* */
/*    Get Division                                                           */
/* ************************************************************************* */

inline int MmlData :: GetDivision(void)
{
	return(_iDivision);
}

/* ************************************************************************* */
/*    Get Bar Count                                                          */
/* ************************************************************************* */

inline int MmlData :: GetBarCount(int iChannel)
{
	return(GetMmlChannel(iChannel)->GetBarCount());
}

/* ************************************************************************* */
/*    Get Step Time                                                          */
/* ************************************************************************* */

inline DWORD MmlData :: GetStepTime(int iChannel)
{
	return(GetMmlChannel(iChannel)->GetStepTime());
}

/* ************************************************************************* */
/*    Get Length Count                                                       */
/* ************************************************************************* */

inline int MmlData :: GetLengthCount(int iChannel, int iLength)
{
	return(GetMmlChannel(iChannel)->GetLengthCount(iLength));
}

/* ************************************************************************* */
/*    Get Max Length Count                                                   */
/* ************************************************************************* */

inline int MmlData :: GetMaxLengthCount(int iChannel)
{
	return(GetMmlChannel(iChannel)->GetMaxLengthCount());
}

/* ************************************************************************* */
/*    Normalize                                                              */
/* ************************************************************************* */

int MmlData :: Normalize(int iDivision, int iBarTime)
{
	int	i;

	for(i = 0; i < GetChannelCount() ; i++){
		GetMmlChannel(i)->Normalize(iDivision / MMLMINDIVRATIO);
		GetMmlChannel(i)->Divide(iBarTime);
		GetMmlChannel(i)->ConvertLength(iDivision);
	}
	return(MMLDATA_COMPLETE);
}

