/* **************************************** */
/*    Registory Interface Class ver.1.01    */
/*                           'regist.cpp'   */
/*                         by T.Nakayama    */
/* **************************************** */

#include "regist.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

Registory :: Registory()
 :	_hKey(0), _hParentKey(0), _lpszSubKey(NULL), _lError(ERROR_SUCCESS),
	_dwDisposition(0), _dwType(0)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

Registory :: ~Registory()
{
}

/* ************************************************************************* */
/*    Set Access Sub Key                                                     */
/* ************************************************************************* */

LONG Registory :: SetAccessSubKey(HKEY hAccessKey, LPCTSTR pSubKey)
{
	_hParentKey = hAccessKey;
	_lpszSubKey = pSubKey;
	return(IsSubKey());
}

/* ************************************************************************* */
/*    Get Access Sub Key                                                     */
/* ************************************************************************* */

inline LPCTSTR Registory :: GetAccessSubKey(PHKEY phAccessKey, LPCTSTR pSubKey)
{
	*phAccessKey = _hParentKey;
	pSubKey = _lpszSubKey;
	return(_lpszSubKey);
}

/* ************************************************************************* */
/*    Get Error                                                              */
/* ************************************************************************* */

inline LONG Registory :: GetError(void)
{
	return(_lError);
}

/* ************************************************************************* */
/*    Get Disposition                                                        */
/* ************************************************************************* */

inline LONG Registory :: GetDisposition(void)
{
	return(_dwDisposition);
}

/* ************************************************************************* */
/*    Get Type                                                               */
/* ************************************************************************* */

inline LONG Registory :: GetType(void)
{
	return(_dwType);
}

/* ************************************************************************* */
/*    Open Sub Key                                                           */
/* ************************************************************************* */

inline LONG Registory :: OpenSubKey(void)
{
	return(RegOpenKeyEx(_hParentKey, _lpszSubKey, 0, KEY_ALL_ACCESS, &_hKey));
}

/* ************************************************************************* */
/*    Create Open Sub Key                                                    */
/* ************************************************************************* */

inline LONG Registory :: CreateOpenSubKey(void)
{
	return(RegCreateKeyEx(_hParentKey, _lpszSubKey, 0, "", 
		REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
		&_hKey, &_dwDisposition));
}

/* ************************************************************************* */
/*    Close Sub Key                                                          */
/* ************************************************************************* */

inline LONG Registory :: CloseSubKey(void)
{
	return(RegCloseKey(_hKey));
}

/* ************************************************************************* */
/*    Is Sub Key                                                             */
/* ************************************************************************* */

LONG Registory :: IsSubKey(void)
{
	_lError = OpenSubKey();
	if(_lError == ERROR_SUCCESS)
		CloseSubKey();
	return(_lError);
}

/* ************************************************************************* */
/*    Create Sub Key                                                         */
/* ************************************************************************* */

LONG Registory :: CreateSubKey(void)
{
	_lError = CreateOpenSubKey();
	if(_lError == ERROR_SUCCESS)
		CloseSubKey();
	return(_lError);
}

/* ************************************************************************* */
/*    Delete Sub Key                                                         */
/* ************************************************************************* */

inline LONG Registory :: DeleteSubKey(void)
{
	return(_lError = RegDeleteKey(_hParentKey, _lpszSubKey));
}

/* ************************************************************************* */
/*    Set DWORD Data                                                         */
/* ************************************************************************* */

LONG Registory :: SetDword(LPCTSTR lpszValueName, DWORD dwValue)
{
	_lError = CreateOpenSubKey();
	if(_lError == ERROR_SUCCESS){
		_lError = RegSetValueEx(_hKey, lpszValueName, 0, REG_DWORD,
			(BYTE *)&dwValue, sizeof(DWORD));
		CloseSubKey();
	}
	return(_lError);
}

/* ************************************************************************* */
/*    Get DWORD Data                                                         */
/* ************************************************************************* */

DWORD Registory :: GetDword(LPCTSTR	lpszValueName)
{
	DWORD	dwSize, dwValue = REGISTORY_ERROR_VALUE;

	_lError = OpenSubKey();
	if(_lError == ERROR_SUCCESS){
		dwSize = sizeof(DWORD);
		_lError = RegQueryValueEx(_hKey, lpszValueName, NULL, &_dwType,
			(LPBYTE)&dwValue, &dwSize);
		CloseSubKey();
	}
	return(dwValue);
}

/* ************************************************************************* */
/*    Get DWORD Data With Default Value                                      */
/* ************************************************************************* */

DWORD Registory :: GetDwordValue(LPCTSTR lpszValueName, DWORD dwDefault)
{
	DWORD	dwValue;

	dwValue = GetDword(lpszValueName);
	if(dwValue == REGISTORY_ERROR_VALUE && GetError() != ERROR_SUCCESS){
		dwValue = dwDefault;
		SetDword(lpszValueName, dwValue);
	}
	return(dwValue);
}


/* ************************************************************************* */
/*    Set String Data                                                        */
/* ************************************************************************* */

LONG Registory :: SetString(LPCTSTR lpszValueName, LPCTSTR lpString)
{
	_lError = CreateOpenSubKey();
	if(_lError == ERROR_SUCCESS){
		_lError = RegSetValueEx(_hKey, lpszValueName, 0, REG_SZ,
			(const LPBYTE)lpString, strlen(lpString));
		CloseSubKey();
	}
	return(_lError);
}

/* ************************************************************************* */
/*    Get String Data                                                        */
/* ************************************************************************* */

DWORD Registory :: GetString(LPCTSTR lpszValueName, LPBYTE lpString, DWORD dwSize)
{
	_lError = OpenSubKey();
	if(_lError == ERROR_SUCCESS){
		_lError = RegQueryValueEx(_hKey, lpszValueName, NULL, &_dwType,
			lpString, &dwSize);
		CloseSubKey();
	}
	return(dwSize);
}

/* ************************************************************************* */
/*    Get String Data With Default Value                                     */
/* ************************************************************************* */

DWORD Registory :: GetStringValue(LPCTSTR lpszValueName, LPBYTE lpString, DWORD dwSize, LPCTSTR lpValue)
{
	dwSize = GetString(lpszValueName, lpString, dwSize);
	if(dwSize == 0 || GetError() != ERROR_SUCCESS){
		SetString(lpszValueName, lpValue);
		strncpy((char *)lpString, lpValue, (size_t)dwSize);
	}
	return(strlen((LPCTSTR)lpString));
}

