/* *************************************** */
/*    Convert SMF to MML Class ver.1.00    */
/*                         'smf2mml.cpp'   */
/*                        by T.Nakayama    */
/* *************************************** */

#include "smf2mml.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

Smf2Mml :: Smf2Mml()
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

Smf2Mml :: ~Smf2Mml()
{
}

/* ************************************************************************* */
/*    Convert SMF Data to MML Data                                           */
/* ************************************************************************* */

int Smf2Mml :: Convert(LPSmfData pSmfData, LPMmlData pMmlData)
{
	int				i, j, iTrackCount, iChannelCount = 0;
	int				iSmfChannel, iMmlChannel = 0;
	DWORD			dwChannel = 0;
	LPSmfTrack		pSmfTrack;
	LPMmlChannel	pMmlChannel;
	MMLCHANNELINFO	stChannelInfo;

	iTrackCount = pSmfData->GetTrackCount();
	for(i=0; i<iTrackCount; i++){
		pSmfTrack = pSmfData->GetSmfTrack(i);
		iChannelCount += GetChannelCount(pSmfTrack);
	}
	pMmlData->AllocateChannel(iChannelCount);
	for(i=0; i<iTrackCount; i++){
		pSmfTrack = pSmfData->GetSmfTrack(i);
		iChannelCount = GetChannelCount(pSmfTrack, &dwChannel);
		for(j=0, iSmfChannel=0; j<iChannelCount; j++){
			pMmlChannel = pMmlData->GetMmlChannel(iMmlChannel);
			while(!(dwChannel & 0x01)){
				dwChannel >>= 1;
				iSmfChannel++;
			}
			stChannelInfo.iSmfTrack   = i;
			stChannelInfo.iSmfChannel = iSmfChannel;
			stChannelInfo.iMmlChannel = iMmlChannel;
			pMmlChannel->SetInfo(&stChannelInfo);
			ConvertChannel(iSmfChannel, pSmfTrack, pMmlChannel);
			iMmlChannel++;
			dwChannel >>= 1;
			iSmfChannel++;
		}
	}
	pMmlData->SetChannelCount(iMmlChannel);
	pMmlData->SetDivision(pSmfData->GetDivision());
	return(iMmlChannel);
}

/* ************************************************************************* */
/*    Convert SMF Track to MML Channel                                       */
/* ************************************************************************* */

int Smf2Mml :: ConvertChannel(
	int iSmfChannel, LPSmfTrack pSmfTrack, LPMmlChannel pMmlChannel)
{
	DWORD				dwDeltaTime = 0;
	LPBYTE				pData;
	LPSmfEvent			pSmfEvent;
	LPMmlEvent			pMmlEvent;
	list < LPSmfEvent > :: iterator	iteSmf, iteSmfEnd;
	BOOL				bEvent;
	UINT				uStatus = SMFSTATUS_REST;
	UINT				uVelocity = 0;
	BYTE				btNoteNumber = MMLCODE_REST;

	iteSmf = pSmfTrack->GetSmfEvent(ITERATOR_BEGIN);
	iteSmfEnd = pSmfTrack->GetSmfEvent(ITERATOR_END);

	bEvent = TRUE;
	while(bEvent && iteSmf != iteSmfEnd){
		pSmfEvent = *iteSmf++;
		pData = pSmfEvent->GetEventData();
		dwDeltaTime += pSmfEvent->GetDeltaTime();
		switch(uStatus){
			case SMFSTATUS_REST:
				switch(*pData){
					case MTRK_EVENT_META:
					case MTRK_EVENT_SYSEX0:
					case MTRK_EVENT_SYSEX7:
						break;
					default:
						if((*pData & 0x0f) == iSmfChannel){
							switch(GetFunctionCode(pData)){
								case MTRK_EVENT_NOTEON:
									pMmlEvent = new MmlEvent;
									pMmlEvent->SetCode(MMLCODE_REST);
									pMmlEvent->SetVelocity(0);
									pMmlEvent->SetDeltaTime(dwDeltaTime);
									pMmlChannel->Add(pMmlEvent);
									btNoteNumber = *(pData + 1);
									uVelocity = (UINT)(*(pData + 2));
									dwDeltaTime = 0;
									uStatus = SMFSTATUS_NOTEON;
									break;
								case MTRK_EVENT_NOTEOFF:
								case MTRK_EVENT_CTRLCHANGE:
								default:
									break;
							}
						}
						break;
				}
				break;

			case SMFSTATUS_NOTEON:
				switch(*pData){
					case MTRK_EVENT_META:
					case MTRK_EVENT_SYSEX0:
					case MTRK_EVENT_SYSEX7:
						break;
					default:
						if((*pData & 0x0f) == iSmfChannel){
							switch(GetFunctionCode(pData)){
								case MTRK_EVENT_NOTEOFF:
									if(btNoteNumber == *(pData + 1)){
										pMmlEvent = new MmlEvent;
										pMmlEvent->SetCode(btNoteNumber);
										pMmlEvent->SetVelocity(uVelocity);
										pMmlEvent->SetDeltaTime(dwDeltaTime);
										pMmlChannel->Add(pMmlEvent);
										btNoteNumber = MMLCODE_REST;
										uVelocity = 0;
										dwDeltaTime = 0;
										uStatus = SMFSTATUS_REST;
									}
									break;
								case MTRK_EVENT_NOTEON:
								case MTRK_EVENT_CTRLCHANGE:
								default:
									break;
							}
						}
						break;
				}
				break;
		}
	}
	return(SMF2MML_COMPLETE);
}

/* ************************************************************************* */
/*    Get Function Code                                                      */
/* ************************************************************************* */

BYTE Smf2Mml :: GetFunctionCode(LPBYTE pData)
{
	BYTE	btCode;

	switch(*pData){
			case MTRK_EVENT_META:
			case MTRK_EVENT_SYSEX0:
			case MTRK_EVENT_SYSEX7:
				btCode = *pData;
				break;
			default:
		switch(*pData & 0xf0){
			case MTRK_EVENT_NOTEON:
				if(*(pData + 2) == 0x00){
					btCode = MTRK_EVENT_NOTEOFF;
					break;
				}
			default:
				btCode = *pData & 0xf0;
				break;
		}
	}
	return(btCode);
}

/* ************************************************************************* */
/*    Get Channel Count                                                      */
/* ************************************************************************* */

int Smf2Mml :: GetChannelCount(LPSmfTrack pSmfTrack, LPDWORD pdwChannel)
{
	int					i, iEventCount, iChannelCount = 0;
	DWORD				dwChannel = 0;
	LPBYTE				pData;
	LPSmfEvent			pSmfEvent;
	list < LPSmfEvent > :: iterator	ite;

	iEventCount = pSmfTrack->GetEventCount();
	ite = pSmfTrack->GetSmfEvent(ITERATOR_BEGIN);
	for(i=0; i<iEventCount; i++,ite++){
		pSmfEvent = *ite;
		pData = pSmfEvent->GetEventData();
		if(0x80 <= *pData && *pData < 0xf0)
			dwChannel |= (1 << (*pData & 0x0f));
	}
	if(pdwChannel)
		*pdwChannel = dwChannel;
	for(i=0; i<16; i++){
		if(dwChannel & 0x01)
			iChannelCount++;
		dwChannel >>= 1;
	}
	return(iChannelCount);
}

