/* *********************************************************** */
/*    Standard MIDI format Block Information Class ver.1.00    */
/*                                            'smfblock.cpp'   */
/*                                            by T.Nakayama    */
/* *********************************************************** */

#include "smfblock.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

SmfBlock :: SmfBlock()
 : 	_iType(SMFBLOCK_NOTSMF)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

SmfBlock :: ~SmfBlock()
{
}

/* ************************************************************************* */
/*    Check Header                                                           */
/* ************************************************************************* */

int SmfBlock :: CheckHeader(LPBYTE pData)
{
	LPSMFBLOCK	pHeader;

	pHeader = (LPSMFBLOCK)pData;
	if(!strncmp(pHeader->chunk, CHUNK_HEADER, strlen(CHUNK_HEADER)))
		return(SMFBLOCK_HEADERBLOCK);
	if(!strncmp(pHeader->chunk, CHUNK_TRACK, strlen(CHUNK_TRACK)))
		return(SMFBLOCK_TRACKBLOCK);
	return(SMFBLOCK_NOTSMF);
}

/* ************************************************************************* */
/*    Read Header                                                            */
/* ************************************************************************* */

int SmfBlock :: ReadHeader(LPBYTE pData)
{
	LPSMFBLOCK		pBlock;

	pBlock = (LPSMFBLOCK)pData;
	_iType = CheckHeader(pData);
	switch(_iType){
		case SMFBLOCK_HEADERBLOCK:
		case SMFBLOCK_TRACKBLOCK:
			_dwLength.SetMSB(pBlock->length);
			break;

		case SMFBLOCK_NOTSMF:
			_dwLength.SetLSB(0);
	}
	return(GetBlockType());
}

/* ************************************************************************* */
/*    Get Header Size                                                        */
/* ************************************************************************* */

inline DWORD SmfBlock :: GetHeaderSize(void)
{
	return(sizeof(SMFBLOCK));
}

/* ************************************************************************* */
/*    Get Block Type                                                         */
/* ************************************************************************* */

inline int SmfBlock :: GetBlockType(void)
{
	return(_iType);
}

/* ************************************************************************* */
/*    Get Block Size                                                         */
/* ************************************************************************* */

inline DWORD SmfBlock :: GetBlockSize(void)
{
	return(_dwLength.GetLSB());
}

