/* ********************************************** */
/*    Standard MIDI Format Data Class ver.1.00    */
/*                                 smfdata.cpp'   */
/*                               by T.Nakayama    */
/* ********************************************** */

#include "smfdata.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

SmfData :: SmfData()
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

SmfData :: ~SmfData()
{
	int		i;

	for(i=0; i<_SmfHeader.GetTrackCount(); i++)
		_DELETE(_pSmfTrack[i]);
}

/* ************************************************************************* */
/*    Is SMF Data?                                                           */
/* ************************************************************************* */

int SmfData :: IsSmfData(LPBYTE pData, DWORD dwSize)
{
	int			i, iTrackCount;
	DWORD		dwCheckSize = 0;
	SmfHeader	smfHeader;
	SmfBlock	smfBlock;

	if(smfHeader.CheckHeader(pData) != SMFBLOCK_HEADERBLOCK)
		return(SMFBLOCK_NOTSMF);
	smfHeader.ReadHeader(pData);
	iTrackCount = smfHeader.GetTrackCount();
	dwCheckSize += (smfHeader.GetHeaderSize() + smfHeader.GetBlockSize());
	pData += (smfHeader.GetHeaderSize() + smfHeader.GetBlockSize());
	for(i=0; i<iTrackCount; i++){
		if(smfBlock.CheckHeader(pData) != SMFBLOCK_TRACKBLOCK)
			return(SMFBLOCK_NOTSMF);
		smfBlock.ReadHeader(pData);
		dwCheckSize += (smfBlock.GetHeaderSize() + smfBlock.GetBlockSize());
		pData += (smfBlock.GetHeaderSize() + smfBlock.GetBlockSize());
	}
	if(dwCheckSize != dwSize)
		return(SMFBLOCK_NOTSMF);
	return(SMFBLOCK_COMPLETE);
}

/* ************************************************************************* */
/*    Read SMF Data                                                          */
/* ************************************************************************* */

int SmfData :: ReadData(LPBYTE pData, DWORD dwSize)
{
	int			i, iTrackCount, iResult;
	SmfBlock	smfTrack;
	DWORD		dwBlockSize;

	if(iResult = IsSmfData(pData, dwSize))
		return(iResult);
	_SmfHeader.ReadHeader(pData);
	iTrackCount = _SmfHeader.GetTrackCount();
	_pSmfTrack.resize(iTrackCount);
	pData += (_SmfHeader.GetHeaderSize() + _SmfHeader.GetBlockSize());
	for(i=0; i<iTrackCount; i++){
		_pSmfTrack[i] = new SmfTrack;
		dwBlockSize = _pSmfTrack[i]->ReadData(pData);
		_pSmfTrack[i]->AssignTime();
		pData += dwBlockSize;
	}
	return(SMFBLOCK_COMPLETE);
}

/* ************************************************************************* */
/*    Get Track Count                                                        */
/* ************************************************************************* */

inline int SmfData :: GetTrackCount(void)
{
	return(_SmfHeader.GetTrackCount());
}

/* ************************************************************************* */
/*    Get Format                                                             */
/* ************************************************************************* */

inline int SmfData :: GetFormat(void)
{
	return(_SmfHeader.GetFormat());
}

/* ************************************************************************* */
/*    Get Division                                                           */
/* ************************************************************************* */

inline int SmfData :: GetDivision(void)
{
	return(_SmfHeader.GetDivision());
}

/* ************************************************************************* */
/*    Get SMF Track                                                          */
/* ************************************************************************* */

LPSmfTrack SmfData :: GetSmfTrack(int iTrack)
{
	if((iTrack < 0) || (iTrack >= _SmfHeader.GetTrackCount()))
		return(NULL);
	else
		return(_pSmfTrack[iTrack]);
}

