/* *********************************************** */
/*    Standard MIDI Format Event Class ver.1.00    */
/*                                'smfevent.cpp'   */
/*                                by T.Nakayama    */
/* *********************************************** */

#include "smfevent.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

SmfEvent :: SmfEvent()
 :	_pEvent(NULL), _dwEventLength(0), _dwTime(0), _dwParam(0)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

SmfEvent :: ~SmfEvent()
{
	_DELETEARRAY(_pEvent);
}

/* ************************************************************************* */
/*    Read Data                                                              */
/* ************************************************************************* */

DWORD SmfEvent :: ReadData(LPBYTE pData)
{
	DWORD			i, dwLength;
	static	BYTE	btRunStatus = 0;

	_deltatime.ReadVarLen(pData);				// Get Event Length
	pData += _deltatime.GetLength();
	_dwEventLength = dwLength = CheckEventLength(pData);

	if(*pData < MTRK_EVENT_RUNNINGSTATUS)		// Running Status
		_dwEventLength++;
	else
		btRunStatus = *pData++;

	_DELETEARRAY(_pEvent);
	_pEvent = new BYTE[_dwEventLength];			// Copy Event Data
	_pEvent[0] = btRunStatus;
	for(i=1; i<_dwEventLength; i++)
		_pEvent[i] = *pData++;

	return(_deltatime.GetLength() + dwLength);
}

/* ************************************************************************* */
/*    Check Event Length                                                     */
/* ************************************************************************* */

DWORD SmfEvent :: CheckEventLength(LPBYTE pData)
{
	BYTE	c;
	DWORD	dwLength = 0;
	VarLen	varlen;

	switch((c = *pData++)){
		case MTRK_EVENT_META:
			varlen.ReadVarLen(++pData);
			dwLength = 2 + varlen.GetLength() + varlen.GetValue();
			break;

		case MTRK_EVENT_SYSEX0:
		case MTRK_EVENT_SYSEX7:
			varlen.ReadVarLen(pData);
			dwLength = 1 + varlen.GetLength() + varlen.GetValue();
			break;

		default:
			if((c >= 0x80 && c <= 0xbf) || (c >= 0xe0 && c<=0xef))
				dwLength = 3;
			else
				dwLength = 2;
	}
	return(dwLength);
}

/* ************************************************************************* */
/*    Get Event Length                                                       */
/* ************************************************************************* */

inline DWORD SmfEvent :: GetEventLength(void)
{
	return(_dwEventLength);
}

/* ************************************************************************* */
/*    Get Event Data                                                         */
/* ************************************************************************* */

inline const LPBYTE SmfEvent :: GetEventData(void)
{
	return(_pEvent);
}

/* ************************************************************************* */
/*    Set Delta Time                                                         */
/* ************************************************************************* */

inline DWORD SmfEvent :: SetDeltaTime(DWORD dwDeltaTime)
{
	return(_deltatime.SetValue(dwDeltaTime));
}

/* ************************************************************************* */
/*    Get Delta Time                                                         */
/* ************************************************************************* */

inline DWORD SmfEvent :: GetDeltaTime(void)
{
	return(_deltatime.GetValue());
}

/* ************************************************************************* */
/*    Set Time                                                               */
/* ************************************************************************* */

DWORD SmfEvent :: SetTime(DWORD dwTime)
{
	DWORD	dwTimeOld;

	dwTimeOld = _dwTime;
	_dwTime = dwTime;
	return(dwTimeOld);
}

/* ************************************************************************* */
/*    Get Time                                                               */
/* ************************************************************************* */

inline DWORD SmfEvent :: GetTime(void)
{
	return(_dwTime);
}

