/* ***************************************************** */
/*    Standard MIDI Format Track Block Class ver.1.00    */
/*                                      'smftrack.cpp'   */
/*                                      by T.Nakayama    */
/* ***************************************************** */

#include "smftrack.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

SmfTrack :: SmfTrack()
 :	_dwTrackTime(0)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

SmfTrack :: ~SmfTrack()
{
	list < LPSmfEvent > :: iterator	ite;

	for(ite=_pSmfEvent.begin(); ite!=_pSmfEvent.end(); ite++)
		_DELETE(*ite);
}

/* ************************************************************************* */
/*    Read Data                                                              */
/* ************************************************************************* */

DWORD SmfTrack :: ReadData(LPBYTE pData)
{
	DWORD				dwEventLength, dwBlockSize = 0;
	LPSmfEvent			pSmfEvent;
	list < LPSmfEvent > :: iterator	ite;

	if(_SmfHeader.CheckHeader(pData) != SMFBLOCK_TRACKBLOCK)
		return(SMFTRACK_NOTTRACKBLOCK);
	_SmfHeader.ReadHeader(pData);
	pData += _SmfHeader.GetHeaderSize();
	ite = _pSmfEvent.begin();
	while(dwBlockSize < _SmfHeader.GetBlockSize()){
		pSmfEvent = new SmfEvent;
		dwEventLength = pSmfEvent->ReadData(pData);
		pData += dwEventLength;
		dwBlockSize += dwEventLength;
		_pSmfEvent.insert(ite, pSmfEvent);
	}
	return(_SmfHeader.GetHeaderSize() + dwBlockSize);
}

/* ************************************************************************* */
/*    Assign Time                                                            */
/* ************************************************************************* */

DWORD SmfTrack :: AssignTime(void)
{
	int					i, iEventCount;
	DWORD				dwDeltaTime, dwTime = 0;
	LPSmfEvent			pSmfEvent;
	list < LPSmfEvent > :: iterator	ite;

	iEventCount = GetEventCount();
	ite = GetSmfEvent(ITERATOR_BEGIN);
	for(i=0; i<iEventCount; i++,ite++){
		pSmfEvent = *ite;
		dwDeltaTime = pSmfEvent->GetDeltaTime();
		dwTime += dwDeltaTime;
		pSmfEvent->SetTime(dwTime);
	}
	_dwTrackTime = dwTime;
	return(dwTime);
}

/* ************************************************************************* */
/*    Get Event Count                                                        */
/* ************************************************************************* */

inline int SmfTrack :: GetEventCount(void)
{
	return(_pSmfEvent.size());
}

/* ************************************************************************* */
/*    Get SMF Event                                                          */
/* ************************************************************************* */

list < LPSmfEvent > :: iterator SmfTrack :: GetSmfEvent(DWORD iType)
{
	list < LPSmfEvent > :: iterator	ite;

	switch(iType){
		case ITERATOR_END:
			ite = _pSmfEvent.end();
			break;

		case ITERATOR_BEGIN:
		default:
			ite = _pSmfEvent.begin();
	}
	return(ite);
}

