/* ****************************************************** */
/*    Window Like Dialogbox Application Class ver.1.03    */
/*                                        'wdlgapp.cpp'   */
/*                                       by T.Nakayama    */
/* ****************************************************** */

#include "wdlgapp.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

WinDlgAppli :: WinDlgAppli()
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

WinDlgAppli :: ~WinDlgAppli()
{
}

/* ************************************************************************* */
/*    Register Window Class                                                  */
/* ************************************************************************* */

ATOM WinDlgAppli :: InitApplication(AppRes *pAppRes)
{
	WNDCLASSEX	wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);			// Size of WNDCLASSEX
	wcex.style = 0;								// Type of window
	wcex.lpfnWndProc = Procedure;				// Address to window proc.
	wcex.cbClsExtra = 0;						// Extra class information
	wcex.cbWndExtra = DLGWINDOWEXTRA;			// Extra window information
	wcex.hInstance = GetInstance();				// Handle to this instance
	wcex.hIcon = pAppRes->GetIcon();			// Handle of Icon
	wcex.hIconSm = pAppRes->GetSmIcon();		// Handle of Small Icon
	wcex.hCursor = pAppRes->GetCursor();		// Handle of mouse cursol
	wcex.hbrBackground = pAppRes->GetBrush();	// Background color
	wcex.lpszMenuName = pAppRes->GetMenuName();	// Name of main menu
	wcex.lpszClassName = GetClassName();		// Name of window class

	return(RegisterClassEx(&wcex));				// Register the window class
}

/* ************************************************************************* */
/*    Create and Show Window                                                 */
/* ************************************************************************* */

HWND WinDlgAppli :: InitInstance(int iCmdShow, LPVOID pVoid, HWND hParentWnd)
{
	HWND	hWnd;

#ifndef APPLI_DIALOGBASE_USE_CLASSNAME
	hWnd = CreateDialog(
		GetInstance(), MAKEINTRESOURCE(IDD_MAIN), hParentWnd, NULL);
#else
	hWnd = CreateDialog(GetInstance(), GetClassName(), hParentWnd, NULL);
#endif

	SendMessage(hWnd, WM_INITDIALOG, 0, (LONG)pVoid);

	if(hWnd){
		ShowWindow(hWnd, iCmdShow);	// Show window
		UpdateWindow(hWnd);			// Update window
	}
	return(hWnd);
}

/* ************************************************************************* */
/*    Message Loop                                                           */
/* ************************************************************************* */

MSG WinDlgAppli :: MessageLoop(void)
{
	MSG		msg;
	HWND	hWnd;
	HACCEL	hAccel;

	while(GetMessage(&msg, NULL, 0, 0)){			// Get message
		hWnd = _pKeyAccel->GetWindowHandle();		// Get Active Win. Handle
		hAccel = _pKeyAccel->GetAcceleratorHandle();// Get Active Accel. Handle
		if(!hWnd || !hAccel || !TranslateAccelerator(hWnd, hAccel, &msg)){
			if(!IsDialogMessage(hWnd, &msg)){
				TranslateMessage(&msg);					// Translate message
				DispatchMessage(&msg);					// Dispatch message
			}
		}
	}

	return(msg);
}

/* ************************************************************************* */
/*    Create Windows Procedure Class                                         */
/* ************************************************************************* */

LPWinProc WinDlgAppli :: CreateProcedure(AppRes *pAppRes, KeyAccel *pKeyAccel)
{
	WinProc	*pWinProc;

	pWinProc = new WinDlgProc;
	if(_pAppli)
		pWinProc->SetAppliClass(_pAppli);
	if(_pAppReg)
		pWinProc->SetAppRegClass(_pAppReg);
	pWinProc->Initialize(pAppRes, pKeyAccel);

	return(pWinProc);
}

/* ************************************************************************* */
/*    Windows Procedure                                          (Static)    */
/* ************************************************************************* */

LRESULT CALLBACK WinDlgAppli :: Procedure(
	HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LONG				lResult;
	LPWinProc			pWinProc;
	LPEXTRAWINDOWINFO	pInfo;
	LPCREATESTRUCT		pCreate;

	switch(uMsg){
		case WM_CREATE:{
				int		iLength;
				LPTSTR	pszTemp;

				pCreate = new CREATESTRUCT;
				*pCreate = *((LPCREATESTRUCT)lParam);
				iLength = strlen(pCreate->lpszName);
				pszTemp = new TCHAR[iLength + 1];
				strcpy(pszTemp, pCreate->lpszName);
				pCreate->lpszName = pszTemp;
				iLength = strlen(pCreate->lpszClass);
				pszTemp = new TCHAR[iLength + 1];
				strcpy(pszTemp, pCreate->lpszClass);
				pCreate->lpszClass = pszTemp;
				SetWindowLong(hWnd, GWL_USERDATA, (LONG)pCreate);
			}
			break;

		case WM_INITDIALOG:{
				LPTSTR	pszTemp;

				pInfo = new EXTRAWINDOWINFO;
				*pInfo = *((LPEXTRAWINDOWINFO)lParam);
				SetWindowLong(hWnd, DWL_USER, (LONG)pInfo);
				pWinProc = (LPWinProc)pInfo->pProc;
				lParam = GetWindowLong(hWnd, GWL_USERDATA);
				lResult = pWinProc->Create(hWnd, 0, lParam);
				pCreate = (LPCREATESTRUCT)lParam;
				pszTemp = (LPTSTR)pCreate->lpszName;
				_DELETEARRAY(pszTemp);
				pszTemp = (LPTSTR)pCreate->lpszClass;
				_DELETEARRAY(pszTemp);
				_DELETE(pCreate);
				SetWindowLong(hWnd, GWL_USERDATA, (LONG)NULL);
			}

		default:
			pInfo = (LPEXTRAWINDOWINFO)GetWindowLong(hWnd, DWL_USER);
			if(pInfo){
				pWinProc = (LPWinProc)pInfo->pProc;
				if(pWinProc){
					lResult = pWinProc->Procedure(hWnd, uMsg, wParam, lParam);
					if(uMsg == WM_DESTROY){
						int			iTable;
						LPWinAppli	pWinAppli;

						pWinAppli = (LPWinAppli)pInfo->pAppli;
						iTable = pWinAppli->FindProc1(pWinProc);
						if(iTable >= 0){
							pWinAppli->DeleteProcPairs(iTable);
							pInfo->pProc = NULL;
						}
						if(pInfo->lQuitMsg)
							PostQuitMessage(0);
						SetWindowLong(hWnd, DWL_USER, (LONG)NULL);
						_DELETE(pInfo);
					}
					return(lResult);
				}
			}
	}
	return(DefWindowProc(hWnd, uMsg, wParam, lParam));
}
