/* **************************************** */
/*    Windows Application Class ver.1.42    */
/*                           'winapp.cpp'   */
/*                         by T.Nakayama    */
/* **************************************** */

#include "winapp.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

WinAppli :: WinAppli()
 :	_pAppRes(NULL), _pKeyAccel(NULL), _pAppli(NULL), _pAppReg(NULL)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

WinAppli :: ~WinAppli()
{
	DestroyAllWindows();
//	_DELETE(_pCmdLine);
	_DELETE(_pAppRes);
}

/* ************************************************************************* */
/*    Set Application Resource Class                                         */
/* ************************************************************************* */

LPAppRes WinAppli :: SetAppResClass(LPAppRes pAppRes)
{
	LPAppRes	pAppResOld;

	pAppResOld = _pAppRes;
	_pAppRes = pAppRes;
	return(pAppResOld);
}

/* ************************************************************************* */
/*    Get Application Resource Class                                         */
/* ************************************************************************* */

inline LPAppRes WinAppli :: GetAppResClass(void)
{
	return(_pAppRes);
}

/* ************************************************************************* */
/*    Set Keyboard Accelerator Class                                         */
/* ************************************************************************* */

LPKeyAccel WinAppli :: SetKeyAccelClass(LPKeyAccel pKeyAccel)
{
	LPKeyAccel	pKeyAccelOld;

	pKeyAccelOld = _pKeyAccel;
	_pKeyAccel = pKeyAccel;
	return(pKeyAccelOld);
}

/* ************************************************************************* */
/*    Get Keyboard Accelerator Class                                         */
/* ************************************************************************* */

inline LPKeyAccel WinAppli :: GetKeyAccelClass(void)
{
	return(_pKeyAccel);
}

/* ************************************************************************* */
/*    Set Application Class                                                  */
/* ************************************************************************* */

LPAppli WinAppli :: SetAppliClass(LPAppli pAppli)
{
	LPAppli	pAppliOld;

	pAppliOld = _pAppli;
	_pAppli = pAppli;
	return(pAppliOld);
}

/* ************************************************************************* */
/*    Get Application Class                                                  */
/* ************************************************************************* */

inline LPAppli WinAppli :: GetAppliClass(void)
{
	return(_pAppli);
}

/* ************************************************************************* */
/*    Set Application Registory Manager Class                                */
/* ************************************************************************* */

LPAppReg WinAppli :: SetAppRegClass(LPAppReg pAppReg)
{
	LPAppReg	pAppRegOld;

	pAppRegOld = _pAppReg;
	_pAppReg = pAppReg;
	return(pAppRegOld);
}

/* ************************************************************************* */
/*    Get Application Registory Manager Class                                */
/* ************************************************************************* */

inline LPAppReg WinAppli :: GetAppRegClass(void)
{
	return(_pAppReg);
}

/* ************************************************************************* */
/*    Initialize                                                             */
/* ************************************************************************* */

int WinAppli :: Initialize(
	HINSTANCE hInst, KeyAccel *pKeyAccel)
{
	TCHAR	szBuf[STRING_BUFFER];

	// ++++++++ Read & Set Class Name & Window Name
	SetInstance(hInst);
	LoadString(hInst, IDS_CLASS, szBuf, STRING_BUFFER);
	SetClassName(szBuf);
	LoadString(hInst, IDS_WINDOW, szBuf, STRING_BUFFER);
	SetWindowName(szBuf);

#ifdef	APPLI_NOT_EXECUTE_SAME_CLASS
	// ++++++++ Search Same Class
	{
		HWND	hTargetWnd;

		if(hTargetWnd = SearchSameClass(GetClassName()))
			return(WINAPPLI_FOUND_SAME_CLASS);
	}
#endif	// APPLI_NOT_EXECUTE_SAME_CLASS

	// ++++++++ Set Keyboard Accelerator Class
	SetKeyAccelClass(pKeyAccel);

	// ++++++++ Create Application Icon, Cursor, Brush, etc.
	_pAppRes = CreateAppRes(hInst);

	// ++++++++ Registory Window Class
	if(!InitApplication(_pAppRes))
		return(WINAPPLI_ERROR_REGIST_CLASS);

	return(WINAPPLI_COMPLETE);
}

/* ************************************************************************* */
/*    Register Window Class                                                  */
/* ************************************************************************* */

ATOM WinAppli :: InitApplication(AppRes *pAppRes)
{
	WNDCLASSEX	wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);			// Size of WNDCLASSEX
	wcex.style = 0;								// Type of window
	wcex.lpfnWndProc = Procedure;				// Address to window proc.
	wcex.cbClsExtra = 0;						// Extra class information
	wcex.cbWndExtra = sizeof(EXTRAWINDOWINFO);	// Extra window information
	wcex.hInstance = GetInstance();				// Handle to this instance
	wcex.hIcon = pAppRes->GetIcon();			// Handle of Icon
	wcex.hIconSm = pAppRes->GetSmIcon();		// Handle of Small Icon
	wcex.hCursor = pAppRes->GetCursor();		// Handle of mouse cursol
	wcex.hbrBackground = pAppRes->GetBrush();	// Background color
	wcex.lpszMenuName = pAppRes->GetMenuName();	// Name of main menu
	wcex.lpszClassName = GetClassName();		// Name of window class

	return(RegisterClassEx(&wcex));				// Register the window class
}

/* ************************************************************************* */
/*    Create and Show Window                                                 */
/* ************************************************************************* */

HWND WinAppli :: InitInstance(int iCmdShow, LPVOID pVoid, HWND hParentWnd)
{
	HWND	hWnd;

#ifdef APPLI_USE_REGISTORY
	hWnd = CreateWindow(			// Create window
		GetClassName(),				// Name of window class
		GetWindowName(),			// Title of window
		WS_OVERLAPPEDWINDOW,		// Style of window
		REG_DEF_WINDOW_X,			// X position of window
		REG_DEF_WINDOW_Y,			// Y position of window
		REG_DEF_WINDOW_W,			// Width of window
		REG_DEF_WINDOW_H,			// Height of window
		hParentWnd,					// Handle of parent window
		NULL,						// Handle of main menu
		GetInstance(),				// Handle of creater
		pVoid						// Pointer to additional information
	);
#else
	hWnd = CreateWindow(			// Create window
		GetClassName(),				// Name of window class
		GetWindowName(),			// Title of window
		WS_OVERLAPPEDWINDOW,		// Style of window
		CW_USEDEFAULT,				// X position of window
		CW_USEDEFAULT,				// Y position of window
		CW_USEDEFAULT,				// Width of window
		CW_USEDEFAULT,				// Height of window
		hParentWnd,					// Handle of parent window
		NULL,						// Handle of main menu
		GetInstance(),				// Handle of creater
		pVoid						// Pointer to additional information
	);
#endif // APPLI_USE_REGISTORY

	if(hWnd){
		ShowWindow(hWnd, iCmdShow);	// Show window
		UpdateWindow(hWnd);			// Update window
	}
	return(hWnd);
}

/* ************************************************************************* */
/*    Message Loop                                                           */
/* ************************************************************************* */

MSG WinAppli :: MessageLoop(void)
{
	MSG		msg;
	HWND	hWnd;
	HACCEL	hAccel;

	while(GetMessage(&msg, NULL, 0, 0)){			// Get message
		hWnd = _pKeyAccel->GetWindowHandle();		// Get Active Win. Handle
		hAccel = _pKeyAccel->GetAcceleratorHandle();// Get Active Accel. Handle
		if(!hWnd || !hAccel || !TranslateAccelerator(hWnd, hAccel, &msg)){
			TranslateMessage(&msg);					// Translate message
			DispatchMessage(&msg);					// Dispatch message
		}
	}

	return(msg);
}

/* ************************************************************************* */
/*    Create Window                                                          */
/* ************************************************************************* */

HWND WinAppli :: CreateWindow_(
	int iCmdShow, LPARAM lParam, LPVOID pVoid, BOOL bQuitMsg, HWND hParentWnd)
{
	HWND				hWnd;
	LPEXTRAWINDOWINFO	pInfo;
	LPPROCPAIRS			pProcPairs;

	pInfo = new EXTRAWINDOWINFO;
	memset(pInfo, 0, sizeof(EXTRAWINDOWINFO));
	pInfo->pAppli = this;
	pInfo->pProc = CreateProcedure(_pAppRes, _pKeyAccel);
	pInfo->lQuitMsg = (LONG)bQuitMsg;
	pInfo->pVoid = pVoid;
	pInfo->lParam = lParam;
	hWnd = InitInstance(iCmdShow, pInfo, hParentWnd);
	pProcPairs = new PROCPAIRS;
	memset(pProcPairs, 0, sizeof(PROCPAIRS));
	pProcPairs->pProc1 = pInfo->pProc;
	pProcPairs->hWnd = hWnd;
	RegisterProcPairs(pProcPairs);
	_DELETE(pProcPairs);
	_DELETE(pInfo);
	return(hWnd);
}

/* ************************************************************************* */
/*    Destroy Window                                                         */
/* ************************************************************************* */

inline BOOL WinAppli :: DestroyWindow(HWND hWnd)
{
	return(:: DestroyWindow(hWnd));
}

/* ************************************************************************* */
/*    Destroy All Windows                                                    */
/* ************************************************************************* */

inline int WinAppli :: DestroyAllWindows(void)
{
	return(SendMessageAllWindows(WM_CLOSE, 0, 0L));
}

/* ************************************************************************* */
/*    Create Application Resource                                            */
/* ************************************************************************* */

LPAppRes WinAppli :: CreateAppRes(HINSTANCE hInst)
{
	AppRes	*pAppRes;
	TCHAR	szMenu[STRING_BUFFER];

	LoadString(_hInst, IDS_MENU, szMenu, STRING_BUFFER);

	pAppRes = new AppRes;

#ifdef APPLI_USE_ORIGINAL_ICON
	pAppRes->SetIconRes(hInst, MAKEINTRESOURCE(IDI_APPLI));
	pAppRes->SetSmIconRes(hInst, MAKEINTRESOURCE(IDI_APPLI));
#else
	pAppRes->SetIconRes(NULL, IDI_APPLICATION);
	pAppRes->SetSmIconRes(NULL, IDI_APPLICATION);
#endif // APPLI_USE_ORIGINAL_ICON

	pAppRes->SetCursorRes(NULL, IDC_ARROW);

#ifdef APPLI_WINDOW_BRUSH
	pAppRes->SetBrush(APPLI_WINDOW_BRUSH);
#else
	pAppRes->SetBrushRes(WHITE_BRUSH);
#endif

	pAppRes->SetMenuName(szMenu);
	pAppRes->SetAcceleratorsRes(hInst, NULL);

	return(pAppRes);
}

/* ************************************************************************* */
/*    Create Windows Procedure Class                                         */
/* ************************************************************************* */

LPWinProc WinAppli :: CreateProcedure(AppRes *pAppRes, KeyAccel *pKeyAccel)
{
	WinProc	*pWinProc;

	pWinProc = new WinProc;
	if(_pAppli)
		pWinProc->SetAppliClass(_pAppli);
	if(_pAppReg)
		pWinProc->SetAppRegClass(_pAppReg);
	pWinProc->Initialize(pAppRes, pKeyAccel);

	return(pWinProc);
}

/* ************************************************************************* */
/*    Search Same Class                                                      */
/* ************************************************************************* */

HWND WinAppli :: SearchSameClass(LPCTSTR pszWClassName)
{
	HWND	hTargetWnd;

	hTargetWnd = FindWindow(pszWClassName, NULL);
	if(hTargetWnd){
		if(IsIconic(hTargetWnd))
			ShowWindow(hTargetWnd, SW_SHOWNORMAL);
		else
			SetForegroundWindow(hTargetWnd);
	}
	return(hTargetWnd);
}

/* ************************************************************************* */
/*    Windows Procedure                                          (Static)    */
/* ************************************************************************* */

LRESULT CALLBACK WinAppli :: Procedure(
	HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LONG		lResult;
	LPWinProc	pWinProc;

	switch(uMsg){
		case WM_CREATE:
			{
				LPCREATESTRUCT		pCreate;
				LPEXTRAWINDOWINFO	pInfo;
				LONG				lQuitMsg, lExtraParam;
				LPVOID				pVoid, pWinAppli;

				pCreate = (CREATESTRUCT *)lParam;
				pInfo = (LPEXTRAWINDOWINFO)pCreate->lpCreateParams;
				pWinAppli = pInfo->pAppli;
				pWinProc = (LPWinProc)pInfo->pProc;
				lQuitMsg = pInfo->lQuitMsg;
				pVoid = pInfo->pVoid;
				lExtraParam = pInfo->lParam;
				SetWindowLong(hWnd, GWL_WINAPPLICLASS, (LONG)pWinAppli);
				SetWindowLong(hWnd, GWL_WINPROCCLASS, (LONG)pWinProc);
				SetWindowLong(hWnd, GWL_QUITMESSAGE, (LONG)lQuitMsg);
				SetWindowLong(hWnd, GWL_PVOID, (LONG)pVoid);
				SetWindowLong(hWnd, GWL_PARAM, (LONG)lExtraParam);
			}

		default:
			pWinProc = (LPWinProc)GetWindowLong(hWnd, GWL_WINPROCCLASS);
			if(pWinProc){
				lResult = pWinProc->Procedure(hWnd, uMsg, wParam, lParam);
				if(uMsg == WM_DESTROY){
					int			iTable;
					LPWinAppli	pWinAppli;

					pWinAppli = (LPWinAppli)GetWindowLong(hWnd,
						GWL_WINAPPLICLASS);
					iTable = pWinAppli->FindProc1(pWinProc);
					if(iTable >= 0){
						pWinAppli->DeleteProcPairs(iTable);
						SetWindowLong(hWnd, GWL_WINPROCCLASS, (LONG)0L);
					}
					if(GetWindowLong(hWnd, GWL_QUITMESSAGE))
						PostQuitMessage(0);
				}
				return(lResult);
			}
	}
	return(DefWindowProc(hWnd, uMsg, wParam, lParam));
}

