/* ************************************** */
/*    Windows Procedure Class ver.1.34    */
/*                        'winproc.cpp'   */
/*                       by T.Nakayama    */
/* ************************************** */

#include "winproc.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

WinProc :: WinProc()
 :	_hAccel(0), _bDelReg(FALSE), _pszHelpPath(NULL), _pCreate(NULL),
 	_pAppRes(NULL), _pKeyAccel(NULL), _pAppli(NULL), _pAppReg(NULL)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

WinProc :: ~WinProc()
{
	FreeHelpPath();
}

/* ************************************************************************* */
/*    Set Application Resource Class                                         */
/* ************************************************************************* */

LPAppRes WinProc :: SetAppResClass(LPAppRes pAppRes)
{
	LPAppRes	pAppResOld;

	pAppResOld = _pAppRes;
	_pAppRes = pAppRes;
	return(pAppResOld);
}

/* ************************************************************************* */
/*    Get Application Resource Class                                         */
/* ************************************************************************* */

inline LPAppRes WinProc :: GetAppResClass(void)
{
	return(_pAppRes);
}

/* ************************************************************************* */
/*    Set Keyboard Accelerator Class                                         */
/* ************************************************************************* */

LPKeyAccel WinProc :: SetKeyAccelClass(LPKeyAccel pKeyAccel)
{
	LPKeyAccel	pKeyAccelOld;

	pKeyAccelOld = _pKeyAccel;
	_pKeyAccel = pKeyAccel;
	return(pKeyAccelOld);
}

/* ************************************************************************* */
/*    Get Keyboard Accelerator Class                                         */
/* ************************************************************************* */

inline LPKeyAccel WinProc :: GetKeyAccelClass(void)
{
	return(_pKeyAccel);
}

/* ************************************************************************* */
/*    Set Application Class                                                  */
/* ************************************************************************* */

LPAppli WinProc :: SetAppliClass(LPAppli pAppli)
{
	LPAppli	pAppliOld;

	pAppliOld = _pAppli;
	_pAppli = pAppli;
	return(pAppliOld);
}

/* ************************************************************************* */
/*    Get Application Class                                                  */
/* ************************************************************************* */

inline LPAppli WinProc :: GetAppliClass(void)
{
	return(_pAppli);
}

/* ************************************************************************* */
/*    Set Application Registory Manager Class                                */
/* ************************************************************************* */

LPAppReg WinProc :: SetAppRegClass(LPAppReg pAppReg)
{
	LPAppReg	pAppRegOld;

	pAppRegOld = _pAppReg;
	_pAppReg = pAppReg;
	return(pAppRegOld);
}

/* ************************************************************************* */
/*    Get Application Registory Manager Class                                */
/* ************************************************************************* */

inline LPAppReg WinProc :: GetAppRegClass(void)
{
	return(_pAppReg);
}

/* ************************************************************************* */
/*    Initialize                                                             */
/* ************************************************************************* */

int WinProc :: Initialize(LPAppRes pAppRes, LPKeyAccel pKeyAccel)
{
	SetAppResClass(pAppRes);
	SetKeyAccelClass(pKeyAccel);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Create Help Path                                                       */
/* ************************************************************************* */

int WinProc :: CreateHelpPath(void)
{
	int		iLength;
	TCHAR	szDir[_MAX_PATH], szFileName[STRING_BUFFER];

	LoadString(_hInst, IDS_HELP_FILENAME, szFileName, STRING_BUFFER);
	GetProgramDirectory(szDir, _MAX_PATH);
	lstrcat(szDir, szFileName);
	iLength = SetHelpPath(szDir);
	return(iLength);
}

/* ************************************************************************* */
/*    Set Help Path                                                          */
/* ************************************************************************* */

int WinProc :: SetHelpPath(LPCTSTR pszBuf)
{
	int	iLength;

	FreeHelpPath();
	iLength = lstrlen(pszBuf);
	_pszHelpPath = new TCHAR[iLength + 1];
	lstrcpy(_pszHelpPath, pszBuf);
	return(iLength);
}

/* ************************************************************************* */
/*    Get Help Path                                                          */
/* ************************************************************************* */

inline LPCTSTR WinProc :: GetHelpPath(void)
{
	return(_pszHelpPath);
}

/* ************************************************************************* */
/*    Free Help Path                                                         */
/* ************************************************************************* */

LPCTSTR WinProc :: FreeHelpPath(void)
{
	_DELETEARRAY(_pszHelpPath);
	return(_pszHelpPath);
}

/* ************************************************************************* */
/*    Get Program Directory                                                  */
/* ************************************************************************* */

int WinProc :: GetProgramDirectory(LPTSTR pszProgDir, int iSize)
{
	LPTSTR	pCmdLine, pDest;
	TCHAR	szPathBuffer[_MAX_PATH];
	TCHAR	szDrive[_MAX_DRIVE];
	TCHAR	szDir[_MAX_DIR];
	TCHAR	szFname[_MAX_FNAME];
	TCHAR	szExt[_MAX_EXT];

	pCmdLine = GetCommandLine();
	if(*pCmdLine == 0x22) pCmdLine++;
	pDest = szPathBuffer;
	while(*pCmdLine != 0x22) *pDest++ = *pCmdLine++;
	*pDest = 0x00;
	_splitpath(szPathBuffer, szDrive, szDir, szFname, szExt);
	wsprintf(szPathBuffer, "%s%s", szDrive, szDir);
	lstrcpyn(pszProgDir, szPathBuffer, iSize);
	return(lstrlen(pszProgDir));
}

/* ************************************************************************* */
/*    Load Window Position                                                   */
/* ************************************************************************* */

int WinProc :: LoadWindowPosition(HWND hWnd, LPAppReg pAppReg)
{
	RECT	rc;
	int		iWidth, iHeight;

	if(!pAppReg)
		return(WINPROC_FALSE);
	pAppReg->GetWindowPosition(&rc);
	if(rc.right != CW_USEDEFAULT && rc.left != CW_USEDEFAULT &&
			rc.top != CW_USEDEFAULT && rc.bottom != CW_USEDEFAULT){
		iWidth = rc.right - rc.left + 0;
		iHeight = rc.bottom - rc.top + 0;
		MoveWindow(hWnd, rc.left, rc.top, iWidth, iHeight, FALSE);
	}
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Save Window Position                                                   */
/* ************************************************************************* */

int WinProc :: SaveWindowPosition(HWND hWnd, LPAppReg pAppReg)
{
	WINDOWPLACEMENT	wp;

	if(!pAppReg)
		return(WINPROC_FALSE);
	memset(&wp, 0, sizeof(WINDOWPLACEMENT));
	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(hWnd, &wp);
	return(pAppReg->SetWindowPosition(&wp.rcNormalPosition));
}

/* ************************************************************************* */
/*    Set Create Struct                                                      */
/* ************************************************************************* */

LPCREATESTRUCT WinProc :: SetCreateStruct(
	HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	LPCREATESTRUCT		pCreate;

	pCreate = (LPCREATESTRUCT)lParam;
	SetWindowHandle(hWnd);
	SetInstance(pCreate->hInstance);
	SetClassName(pCreate->lpszClass);
	SetWindowName(pCreate->lpszName);
	return(pCreate);
}

/* ************************************************************************* */
/*    Active Keyboard Accelerator                                            */
/* ************************************************************************* */

HWND WinProc :: ActiveKeyAccelerator(
	HWND hWnd, LPKeyAccel pKeyAccel, HACCEL hAccel)
{
	HWND	hWndOld;

	hWndOld = pKeyAccel->SetWindowHandle(hWnd);
	pKeyAccel->SetAcceleratorHandle(hAccel);
	return(hWndOld);
}

/* ************************************************************************* */
/*    Create Menu Help Pairs                                                 */
/* ************************************************************************* */

LPMENUPAIRS WinProc :: CreateMenuHelpPairs(int iMenuItemMax)
{
	LPMENUPAIRS		pIDs;

	pIDs = new MENUPAIRS[MENUITEM_MAX];
	memset(pIDs, 0, sizeof(MENUPAIRS) * MENUITEM_MAX);
	return(pIDs);
}

/* ************************************************************************* */
/*    Delete Menu Help Pairs                                                 */
/* ************************************************************************* */

LPMENUPAIRS WinProc :: DeleteMenuHelpPairs(LPMENUPAIRS pIDs)
{
	_DELETE(pIDs);
	return(pIDs);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_DELREGISTORY                                          */
/* ************************************************************************* */

int WinProc :: CmdDelRegistory(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int		iRet;
	TCHAR	szMsg[STRING_BUFFER];

	if(!_bDelReg){
		LoadString(_hInst, IDS_DELETE_REGISTORY, szMsg, STRING_BUFFER);
		iRet = MessageBox(hWnd, szMsg, GetWindowName(),
			MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2);
		if(iRet != IDYES) return(WINPROC_CANCEL);
	}
	_bDelReg = _bDelReg ? FALSE : TRUE;
	return(_bDelReg ? WINPROC_TRUE : WINPROC_FALSE);
}

/* ************************************************************************* */
/*    WM_COMMAND : IDM_ABOUT                                                 */
/* ************************************************************************* */

int WinProc :: CmdAbout(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	LPDlgAboutAppli	pDlgAboutAppli;

	pDlgAboutAppli = new DlgAboutAppli;
	pDlgAboutAppli->CreateDialog_(_hInst, hWnd);
	_DELETE(pDlgAboutAppli);
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_COMMAND                                                             */
/* ************************************************************************* */

int WinProc :: Command(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam)){

		case IDM_DELREGISTORY:	// ---------------------- IDM_DELREGISTORY ----
			CmdDelRegistory(hWnd, wParam, lParam);
			break;

		case IDM_ABOUT:	// ------------------------------------- IDM_ABOUT ----
			CmdAbout(hWnd, wParam, lParam);
			break;

#ifdef	APPLI_USE_HELP
		case IDM_HELP:	// -------------------------------------- IDM_HELP ----
			WinHelp(hWnd, _pszHelpPath, HELP_FINDER, 0);
			break;
#endif

		case IDM_EXIT:	// -------------------------------------- IDM_EXIT ----
			DestroyWindow(hWnd);
			break;
	}
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_ACTIVATE                                                            */
/* ************************************************************************* */

int WinProc :: Activate(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam)){
		case WA_ACTIVE:
		case WA_CLICKACTIVE:
			_pKeyAccel->SetWindowHandle(hWnd);
			_pKeyAccel->SetAcceleratorHandle(_hAccel);
			break;
	}
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_INITMENU                                                            */
/* ************************************************************************* */

int WinProc :: InitMenu(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	HMENU	hMenu;

	hMenu = (HMENU)wParam;
	CheckMenuItem(hMenu, IDM_DELREGISTORY,
		MF_BYCOMMAND | _bDelReg ? MF_CHECKED : MF_UNCHECKED);

	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_SIZE                                                                */
/* ************************************************************************* */

int WinProc :: Size(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_CREATE                                                              */
/* ************************************************************************* */

int WinProc :: Create(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int		iResult;

	// ======== Set Create Struct ========
	_pCreate = SetCreateStruct(hWnd, wParam, lParam);

	// ======== Set Keyboard Accelerator Handle ========
	_hAccel = _pAppRes->GetAccelerators();
	ActiveKeyAccelerator(hWnd, _pKeyAccel, _hAccel);

#ifdef	APPLI_USE_HELP
	// ======== Create Help Path ========
	CreateHelpPath();
#endif

	// ======== Load Window Position ========
	if(_pAppReg)
		LoadWindowPosition(hWnd, _pAppReg);

	// ======== WM_CREATE Appendix ========
	iResult = CreateAppendix(hWnd, wParam, lParam);

	return(iResult);
}

/* ************************************************************************* */
/*    WM_CREATE Appendix                                                     */
/* ************************************************************************* */

int WinProc :: CreateAppendix(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_DESTROY                                                             */
/* ************************************************************************* */

int WinProc :: Destroy(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int		iResult;

	// ======== WM_DESTROY Appendix ========
	iResult = DestroyAppendix(hWnd, wParam, lParam);

	// ======== Save Window Position ========
	if(_pAppReg)
		SaveWindowPosition(hWnd, _pAppReg);

	// ======== Delete Registory ========
	if(_pAppReg && _bDelReg)
		_pAppReg->DeleteRegistory();

	// ======== Free Window Information ========
	SetInstance(0);
	FreeClassName();
	FreeWindowName();

	return(iResult);
}

/* ************************************************************************* */
/*    WM_DESTROY Appendix                                                    */
/* ************************************************************************* */

int WinProc :: DestroyAppendix(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    WM_CLOSE                                                               */
/* ************************************************************************* */

int WinProc :: Close(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	return(WINPROC_COMPLETE);
}

/* ************************************************************************* */
/*    Window Procedure                                                       */
/* ************************************************************************* */

LRESULT WinProc :: Procedure(
	HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message){
		case WM_COMMAND:	// ++++++++++++++++++++++++++++++++ WM_COMMAND ++++
			Command(hWnd, wParam, lParam);
			break;

		case WM_ACTIVATE:	// +++++++++++++++++++++++++++++++ WM_ACTIVATE ++++
			Activate(hWnd, wParam, lParam);
			break;

		case WM_INITMENU:	// +++++++++++++++++++++++++++++++ WM_INITMENU ++++
			InitMenu(hWnd, wParam, lParam);
			break;

		case WM_SIZE:	// +++++++++++++++++++++++++++++++++++++++ WM_SIZE ++++
			Size(hWnd, wParam, lParam);
			break;

		case WM_CREATE:	// +++++++++++++++++++++++++++++++++++++ WM_CREATE ++++
			Create(hWnd, wParam, lParam);
			break;

		case WM_DESTROY:	// ++++++++++++++++++++++++++++++++ WM_DESTROY ++++
			Destroy(hWnd, wParam, lParam);
			break;

		case WM_CLOSE:	// ++++++++++++++++++++++++++++++++++++++ WM_CLOSE ++++
			Close(hWnd, wParam, lParam);
			DestroyWindow(hWnd);
			break;

		default:
			// Let Window 95 process any message not specified
			// in the precesing swtich statement.
			return(DefWindowProc(hWnd, message, wParam, lParam));
	}
	return(0);
}

