/* ********************************* */
/*    Windows Base Class ver.1.00    */
/*                  'winsbase.cpp'   */
/*                  by T.Nakayama    */
/* ********************************* */

#include "winsbase.h"

/* ************************************************************************* */
/*    Constructor                                                            */
/* ************************************************************************* */

WindowsBase :: WindowsBase()
 :	_hWnd(0), _hInst(0), _hParentWnd(0),
	_pszWClassName(NULL), _pszWindowName(NULL)
{
}

/* ************************************************************************* */
/*    Destructor                                                             */
/* ************************************************************************* */

WindowsBase :: ~WindowsBase()
{
	FreeWindowName();
	FreeClassName();
}

/* ************************************************************************* */
/*    Set Window Handle                                                      */
/* ************************************************************************* */

HWND WindowsBase :: SetWindowHandle(HWND hWnd)
{
	HWND	hWndOld;

	hWndOld = _hWnd;
	_hWnd = hWnd;
	return(hWndOld);
}

/* ************************************************************************* */
/*    Get Window Handle                                                      */
/* ************************************************************************* */

inline HWND WindowsBase :: GetWindowHandle(void)
{
	return(_hWnd);
}

/* ************************************************************************* */
/*    Set Parent Window Handle                                               */
/* ************************************************************************* */

HWND WindowsBase :: SetParentWindowHandle(HWND hParentWnd)
{
	HWND	hParentWndOld;

	hParentWndOld = _hParentWnd;
	_hParentWnd = hParentWnd;
	return(hParentWndOld);
}

/* ************************************************************************* */
/*    Get Parent Window Handle                                               */
/* ************************************************************************* */

inline HWND WindowsBase :: GetParentWindowHandle(void)
{
	return(_hParentWnd);
}

/* ************************************************************************* */
/*    Set Instance                                                           */
/* ************************************************************************* */

HINSTANCE WindowsBase :: SetInstance(HINSTANCE hInst)
{
	HINSTANCE	hInstOld;

	hInstOld = _hInst;
	_hInst = hInst;
	return(hInstOld);
}

/* ************************************************************************* */
/*    Get Instance                                                           */
/* ************************************************************************* */

inline HINSTANCE WindowsBase :: GetInstance(void)
{
	return(_hInst);
}

/* ************************************************************************* */
/*    Set Window Class Name                                                  */
/* ************************************************************************* */

int WindowsBase :: SetClassName(LPCTSTR pszBuf)
{
	int	iLength;

	FreeClassName();
	iLength = lstrlen(pszBuf);
	_pszWClassName = new TCHAR[iLength + 1];
	lstrcpy(_pszWClassName, pszBuf);
	return(iLength);
}

/* ************************************************************************* */
/*    Get Window Class Name                                                  */
/* ************************************************************************* */

inline LPCTSTR WindowsBase :: GetClassName(void)
{
	return(_pszWClassName);
}

/* ************************************************************************* */
/*    Free Window Class Name                                                 */
/* ************************************************************************* */

LPCTSTR WindowsBase :: FreeClassName(void)
{
	_DELETEARRAY(_pszWClassName);
	return(_pszWClassName);
}

/* ************************************************************************* */
/*    Set Window Name                                                        */
/* ************************************************************************* */

int WindowsBase :: SetWindowName(LPCTSTR pszBuf)
{
	int	iLength;

	FreeWindowName();
	iLength = lstrlen(pszBuf);
	_pszWindowName = new TCHAR[iLength + 1];
	lstrcpy(_pszWindowName, pszBuf);
	return(iLength);
}

/* ************************************************************************* */
/*    Get Window Name                                                        */
/* ************************************************************************* */

inline LPCTSTR WindowsBase :: GetWindowName(void)
{
	return(_pszWindowName);
}

/* ************************************************************************* */
/*    Free Window Name                                                       */
/* ************************************************************************* */

LPCTSTR WindowsBase :: FreeWindowName(void)
{
	_DELETEARRAY(_pszWindowName);
	return(_pszWindowName);
}

